@@ -61,6 +61,8 @@
 #include "ms-excel-xf.h"
 #include "ms-formula-write.h"
 
+#define N_ELEMENTS_BETWEEN_PROGRESS_UPDATES   20
+
 static excel_iconv_t current_workbook_iconv = NULL;
 /**
  *  This function writes simple strings...
@@ -288,6 +290,23 @@ write_magic_interface (BiffPut *bp, MsBiffVersion ver)
 	}
 }
 
+/* See: S59DE3.HTM */
+static void
+write_setup (BiffPut *bp, ExcelSheet *sheet)
+{
+	guint8 * data = ms_biff_put_len_next (bp, BIFF_SETUP, 34);
+	MS_OLE_SET_GUINT32 (data +  0, 0x002c0000);
+	MS_OLE_SET_GUINT32 (data +  4, 0x00010001);
+	MS_OLE_SET_GUINT32 (data +  8, 0x00440001);
+	MS_OLE_SET_GUINT32 (data + 12, 0x0000002f);
+	MS_OLE_SET_GUINT32 (data + 16, 0x00000000);
+	MS_OLE_SET_GUINT32 (data + 20, 0x3fe00000);
+	MS_OLE_SET_GUINT32 (data + 24, 0x00000000);
+	MS_OLE_SET_GUINT32 (data + 28, 0x3fe00000);
+	MS_OLE_SET_GUINT16 (data + 32, excel_iconv_win_codepage());
+	ms_biff_put_commit (bp);
+}
+
 /*
  * XL doesn't write externsheets as often as we do. The previous version of
  * this function causes XL to crash when we exported a spreadsheet function
@@ -398,14 +417,14 @@ write_window2 (BiffPut *bp, MsBiffVersion ver, ExcelSheet *sheet)
 		options |= 0x600; /* assume selected if it is current */
 
 	if (ver <= MS_BIFF_V7) {
-		data = ms_biff_put_len_next (bp, BIFF_WINDOW2, 10);
+		data = ms_biff_put_len_next (bp, 0x200|BIFF_WINDOW2, 10);
 
 		MS_OLE_SET_GUINT16 (data +  0, options);
 		MS_OLE_SET_GUINT16 (data +  2, 0x0);	/* top row */
 		MS_OLE_SET_GUINT16 (data +  4, 0x0);	/* left col */
 		MS_OLE_SET_GUINT32 (data +  6, 0x40);	/* grid color index (0x40 == auto) */
 	} else {
-		data = ms_biff_put_len_next (bp, BIFF_WINDOW2, 18);
+		data = ms_biff_put_len_next (bp, 0x200|BIFF_WINDOW2, 18);
 
 		MS_OLE_SET_GUINT16 (data +  0, options);
 		MS_OLE_SET_GUINT16 (data +  2, 0x0);	/* top row */
@@ -1580,6 +1599,8 @@ pre_cell (gconstpointer dummy, Cell *cell, ExcelSheet *sheet)
 	g_return_if_fail (cell != NULL);
 	g_return_if_fail (sheet != NULL);
 
+	count_io_progress_update (sheet->wb->io_context, 1);
+
 	col = cell->pos.col;
 	row = cell->pos.row;
 
@@ -1644,10 +1665,12 @@ pre_blanks (ExcelSheet *sheet)
 {
 	int row, col;
 
-	for (row = 0; row < sheet->maxy; row++)
+	for (row = 0; row < sheet->maxy; row++) {
 		for (col = 0; col < sheet->maxx; col++)
 			if (!cell_is_used (sheet, col, row))
 				pre_blank (sheet, col, row);
+		count_io_progress_update (sheet->wb->io_context, 1);
+	}
 }
 
 /**
@@ -1672,8 +1695,18 @@ pre_blanks (ExcelSheet *sheet)
 static void
 gather_styles (ExcelWorkbook *wb)
 {
-	guint i;
+	guint n = 0, i;
+	enum {N_CELLS_BETWEEN_UPDATES = 20};
 
+	for (i = 0; i < wb->sheets->len; i++) {
+		ExcelSheet *s;
+
+		s = g_ptr_array_index (wb->sheets, i);
+		n += g_hash_table_size (s->gnum_sheet->cell_hash);
+		n += s->maxy;
+	}	
+
+	count_io_progress_set (wb->io_context, n, N_CELLS_BETWEEN_UPDATES);
 	for (i = 0; i < wb->sheets->len; i++) {
 		ExcelSheet *s = g_ptr_array_index (wb->sheets, i);
 
@@ -1682,6 +1715,7 @@ gather_styles (ExcelWorkbook *wb)
 				      (GHFunc) pre_cell, s);
 		pre_blanks (s);
 	}
+	io_progress_unset (wb->io_context);
 }
 
 /**
@@ -2283,7 +2317,7 @@ write_xf (BiffPut *bp, ExcelWorkbook *wb)
 
 	/* See: S59DEA.HTM */
 	for (lp = 0; lp < 6; lp++) {
-		guint8 *data = ms_biff_put_len_next (bp, BIFF_STYLE, 4);
+		guint8 *data = ms_biff_put_len_next (bp, 0x200|BIFF_STYLE, 4);
 		MS_OLE_SET_GUINT32 (data, style_magic[lp]); /* cop out */
 		ms_biff_put_commit (bp);
 	}
@@ -2321,7 +2355,7 @@ write_names (BiffPut *bp, ExcelWorkbook *wb)
 		for (i = 0; i < 20; i++) data[i] = 0;
 
 		text = expr_name->name->str;
-		ms_biff_put_var_next (bp, BIFF_NAME);
+		ms_biff_put_var_next (bp, 0x200|BIFF_NAME);
 		name_len = strlen (expr_name->name->str);
 		MS_OLE_SET_GUINT8 (data + 3, name_len); /* name_len */
 
@@ -3021,6 +3055,29 @@ write_colinfo (BiffPut *bp, ExcelSheet *sheet,
 	ms_biff_put_commit (bp);
 }
 
+/* See: S59D76.HTM */
+static void
+write_dimension (BiffPut *bp, ExcelSheet *sheet)
+{
+	guint8 *data;
+	if (sheet->wb->ver >= MS_BIFF_V8) {
+		data = ms_biff_put_len_next (bp, 0x200 | BIFF_DIMENSIONS, 14);
+		MS_OLE_SET_GUINT32 (data +  0, 0);
+		MS_OLE_SET_GUINT32 (data +  4, sheet->maxy);
+		MS_OLE_SET_GUINT16 (data +  8, 0);
+		MS_OLE_SET_GUINT16 (data + 10, sheet->maxx);
+		MS_OLE_SET_GUINT16 (data + 12, 0x0000);
+	} else {
+		data = ms_biff_put_len_next (bp, BIFF_DIMENSIONS, 10);
+		MS_OLE_SET_GUINT16 (data +  0, 0);
+		MS_OLE_SET_GUINT16 (data +  2, sheet->maxy);
+		MS_OLE_SET_GUINT16 (data +  4, 0);
+		MS_OLE_SET_GUINT16 (data +  6, sheet->maxx);
+		MS_OLE_SET_GUINT16 (data +  8, 0x0000);
+	}
+	ms_biff_put_commit (bp);
+}
+
 /**
  * write_colinfos
  * @bp     BIFF buffer
@@ -3050,11 +3107,12 @@ write_colinfos (BiffPut *bp, ExcelSheet *sheet)
 }
 
 static void
-write_sheet_bools (BiffPut *bp, ExcelSheet *sheet)
+write_sheet_head (BiffPut *bp, ExcelSheet *sheet)
 {
 	guint8 *data;
 	PrintInformation *pi;
 	MsBiffVersion ver = sheet->wb->ver;
+	Workbook *wb = sheet->gnum_sheet->workbook;
 
 	g_return_if_fail (sheet != NULL);
 	g_return_if_fail (sheet->gnum_sheet != NULL);
@@ -3069,7 +3127,7 @@ write_sheet_bools (BiffPut *bp, ExcelSheet *sheet)
 
 	/* See: S59D62.HTM */
 	data = ms_biff_put_len_next (bp, BIFF_CALCCOUNT, 2);
-	MS_OLE_SET_GUINT16 (data, 0x0064);
+	MS_OLE_SET_GUINT16 (data, wb->iteration.max_number);
 	ms_biff_put_commit (bp);
 
 	/* See: S59DD7.HTM */
@@ -3079,13 +3137,12 @@ write_sheet_bools (BiffPut *bp, ExcelSheet *sheet)
 
 	/* See: S59D9C.HTM */
 	data = ms_biff_put_len_next (bp, BIFF_ITERATION, 2);
-	MS_OLE_SET_GUINT16 (data, 0x0000);
+	MS_OLE_SET_GUINT16 (data, wb->iteration.enabled ? 1 : 0);
 	ms_biff_put_commit (bp);
 
-	/* See: S59D75.HTM, FIXME: find what number this really is! */
+	/* See: S59D75.HTM */
 	data = ms_biff_put_len_next (bp, BIFF_DELTA, 8);
-	MS_OLE_SET_GUINT32 (data,   0xd2f1a9fc);
-	MS_OLE_SET_GUINT32 (data+4, 0x3f50624d);
+	gnumeric_set_le_double (data, wb->iteration.tolerance);
 	ms_biff_put_commit (bp);
 
 	/* See: S59DDD.HTM */
@@ -3147,44 +3204,12 @@ write_sheet_bools (BiffPut *bp, ExcelSheet *sheet)
 	margin_write (bp, BIFF_TOP_MARGIN,    &pi->margins.top);
 	margin_write (bp, BIFF_BOTTOM_MARGIN, &pi->margins.bottom);
 
-	/* See: S59DE3.HTM */
-	data = ms_biff_put_len_next (bp, BIFF_SETUP, 34);
-	MS_OLE_SET_GUINT32 (data +  0, 0x002c0000);
-	MS_OLE_SET_GUINT32 (data +  4, 0x00010001);
-	MS_OLE_SET_GUINT32 (data +  8, 0x00440001);
-	MS_OLE_SET_GUINT32 (data + 12, 0x0000002f);
-	MS_OLE_SET_GUINT32 (data + 16, 0x00000000);
-	MS_OLE_SET_GUINT32 (data + 20, 0x3fe00000);
-	MS_OLE_SET_GUINT32 (data + 24, 0x00000000);
-	MS_OLE_SET_GUINT32 (data + 28, 0x3fe00000);
-	MS_OLE_SET_GUINT16 (data + 32, excel_iconv_win_codepage());
-	ms_biff_put_commit (bp);
-
+	write_setup (bp, sheet);
 	write_externsheets (bp, sheet->wb, sheet);
-
 	ms_formula_write_pre_data (bp, sheet, EXCEL_EXTERNNAME, ver);
-
-	write_default_col_width (bp, sheet); /* Default column width */
-
-	write_colinfos (bp, sheet); /* Column infos */
-
-	/* See: S59D76.HTM */
-	if (ver >= MS_BIFF_V8) {
-		data = ms_biff_put_len_next (bp, BIFF_DIMENSIONS, 14);
-		MS_OLE_SET_GUINT32 (data +  0, 0);
-		MS_OLE_SET_GUINT32 (data +  4, sheet->maxy);
-		MS_OLE_SET_GUINT16 (data +  8, 0);
-		MS_OLE_SET_GUINT16 (data + 10, sheet->maxx);
-		MS_OLE_SET_GUINT16 (data + 12, 0x0000);
-	} else {
-		data = ms_biff_put_len_next (bp, (0x200 | BIFF_DIMENSIONS), 10);
-		MS_OLE_SET_GUINT16 (data +  0, 0);
-		MS_OLE_SET_GUINT16 (data +  2, sheet->maxy);
-		MS_OLE_SET_GUINT16 (data +  4, 0);
-		MS_OLE_SET_GUINT16 (data +  6, sheet->maxx);
-		MS_OLE_SET_GUINT16 (data +  8, 0x0000);
-	}
-	ms_biff_put_commit (bp);
+	write_default_col_width (bp, sheet);
+	write_colinfos (bp, sheet);
+	write_dimension (bp, sheet);
 }
 
 static void
@@ -3401,6 +3426,7 @@ write_block (BiffPut *bp, ExcelSheet *sheet, guint32 begin, int nrows)
 					run_size = 0;
 				}
 				write_cell (bp, sheet, cell);
+				workbook_io_progress_update (sheet->wb->io_context, 1);
 			}
 		}
 		if (run_size > 0 && run_size <= maxx) {
@@ -3457,7 +3483,7 @@ write_sheet (IOContext *context, BiffPut *bp, ExcelSheet *sheet)
 	}
 	ms_biff_put_commit (bp);
 
-	write_sheet_bools (bp, sheet);
+	write_sheet_head (bp, sheet);
 
 #ifndef NO_DEBUG_EXCEL
 	if (ms_excel_write_debug > 1)
@@ -3544,7 +3570,7 @@ pre_pass (IOContext *context, ExcelWorkbook *wb)
 	put_font (wb->xf->default_style, NULL, wb);
 	put_format (wb->xf->default_style, NULL, wb);
 
-	gather_styles (wb);	/* (and cache cells) */
+	gather_styles (wb);     /* (and cache cells) */
 	/* Gather Info from styles */
 	gather_fonts (wb);
 	gather_formats (wb);
@@ -3608,10 +3634,13 @@ write_workbook (IOContext *context, BiffPut *bp, ExcelWorkbook *wb, MsBiffVersio
 	biff_eof_write (bp);
 	/* End of Workbook */
 
+	workbook_io_progress_set (context, wb->gnum_wb, WB_PROGRESS_CELLS,
+	                          N_ELEMENTS_BETWEEN_PROGRESS_UPDATES);
 	/* Sheets */
 	for (lp = 0; lp < wb->sheets->len; lp++)
 		write_sheet (context, bp, g_ptr_array_index (wb->sheets, lp));
 	/* End of Sheets */
+	io_progress_unset (context);
 
 	/* Finalise Workbook stuff */
 	for (lp = 0; lp < wb->sheets->len; lp++) {
@@ -3678,6 +3707,7 @@ ms_excel_check_write (IOContext *context, void **state, WorkbookView *gwb_view,
 	*state = wb;
 
 	wb->ver          = ver;
+	wb->io_context   = context;
 	wb->gnum_wb      = wb_view_workbook (gwb_view);
 	wb->gnum_wb_view = gwb_view;
 	wb->sheets   = g_ptr_array_new ();
