@@ -39,6 +39,7 @@
 #include "cell.h"
 #include "position.h"
 #include "expr.h"
+#include "expr-name.h"
 #include "print-info.h"
 #include "value.h"
 #include "selection.h"
@@ -55,8 +56,9 @@
 
 GNUMERIC_MODULE_PLUGIN_INFO_DECL;
 
-gboolean xml_sax_file_probe (FileOpener const *fo, const gchar *file_name);
-void     xml_sax_file_open (FileOpener const *fo, IOContext *io_context,
+gboolean xml_sax_file_probe (GnumFileOpener const *fo, const gchar *file_name,
+                             FileProbeLevel pl);
+void     xml_sax_file_open (GnumFileOpener const *fo, IOContext *io_context,
 			    WorkbookView *wb_view, char const *filename);
 
 /*****************************************************************************/
@@ -209,6 +211,12 @@ STATE_WB,
 			STATE_WB_SUMMARY_ITEM_NAME,
 			STATE_WB_SUMMARY_ITEM_VALUE_STR,
 			STATE_WB_SUMMARY_ITEM_VALUE_INT,
+	STATE_WB_SHEETNAME_INDEX,
+		STATE_WB_SHEETNAME,
+        STATE_NAMES,
+                STATE_NAMES_NAME,
+                        STATE_NAMES_NAME_NAME,
+                        STATE_NAMES_NAME_VALUE,
 	STATE_WB_GEOMETRY,
 	STATE_WB_SHEETS,
 		STATE_SHEET,
@@ -216,6 +224,10 @@ STATE_WB,
 			STATE_SHEET_MAXCOL,	/* convert to attr */
 			STATE_SHEET_MAXROW,	/* convert to attr */
 			STATE_SHEET_ZOOM,	/* convert to attr */
+			STATE_SHEET_NAMES,
+				STATE_SHEET_NAMES_NAME,
+					STATE_SHEET_NAMES_NAME_NAME,
+					STATE_SHEET_NAMES_NAME_VALUE,
 			STATE_SHEET_PRINTINFO,
                                 STATE_PRINT_MARGINS,
 					STATE_PRINT_MARGIN_TOP,
@@ -262,13 +274,12 @@ STATE_WB,
 				STATE_SHEET_MERGE,
 			STATE_SHEET_SOLVER,
 		STATE_SHEET_OBJECTS,
-				STATE_OBJECT_POINTS,
+			STATE_OBJECT_POINTS,
 			STATE_OBJECT_RECTANGLE,
 			STATE_OBJECT_ELLIPSE,
 			STATE_OBJECT_ARROW,
 			STATE_OBJECT_LINE,
 
-	STATE_NAMES,
 	STATE_WB_VIEW,
 
 	STATE_UNKNOWN
@@ -293,6 +304,12 @@ static char const * const xmlSax_state_names[] =
 			"gmr:name",
 			"gmr:val-string",
 			"gmr:val-int",
+	"gmr:SheetNameIndex",
+		"gmr:SheetName",
+        "gmr:Names",
+                "gmr:Name",
+                        "gmr:name",
+                        "gmr:value",
 	"gmr:Geometry",
 	"gmr:Sheets",
 		"gmr:Sheet",
@@ -300,6 +317,10 @@ static char const * const xmlSax_state_names[] =
 			"gmr:MaxCol",
 			"gmr:MaxRow",
 			"gmr:Zoom",
+			"gmr:Names",
+				"gmr:Name",
+					"gmr:name",
+					"gmr:value",
 			"gmr:PrintInformation",
 				"gmr:Margins",
 					"gmr:top",
@@ -347,12 +368,11 @@ static char const * const xmlSax_state_names[] =
 				"gmr:Merge",
 			"gmr:Solver",
 			"gmr:Objects",
-					"gmr:Points",
+				"gmr:Points",
 				"gmr:Rectangle",
 				"gmr:Ellipse",
 				"gmr:Arrow",
 				"gmr:Line",
-	"gmr:Names",
 	"gmr:UIData",
 
 	"Unknown",
@@ -368,9 +388,10 @@ typedef enum
     GNUM_XML_V4,	/* >= 0.57 */
     GNUM_XML_V5,	/* >= 0.58 */
     GNUM_XML_V6,	/* >= 0.62 */
+    GNUM_XML_V7,        /* >= 0.66 */
 
     /* NOTE : Keep this up to date */
-    GNUM_XML_LATEST = GNUM_XML_V6
+    GNUM_XML_LATEST = GNUM_XML_V7
 } GnumericXMLVersion;
 typedef struct _XMLSaxParseState
 {
@@ -394,6 +415,12 @@ typedef struct _XMLSaxParseState
 	} attribute;
 	GList *attributes;
 
+	/* Only valid when parsing wb or sheet names */
+	struct {
+		char *name;
+		char *value;
+	} name;
+	
 	gboolean  style_range_init;
 	Range	  style_range;
 	MStyle   *style;
@@ -471,6 +498,7 @@ xml_sax_wb (XMLSaxParseState *state, CHAR const **attrs)
 				char const * const id;
 				GnumericXMLVersion const version;
 			} GnumericVersions [] = {
+				{ "http://www.gnome.org/gnumeric/v7", GNUM_XML_V7 },	/* 0.66 */
 				{ "http://www.gnome.org/gnumeric/v6", GNUM_XML_V6 },	/* 0.62 */
 				{ "http://www.gnome.org/gnumeric/v5", GNUM_XML_V5 },
 				{ "http://www.gnome.org/gnumeric/v4", GNUM_XML_V4 },
@@ -481,17 +509,27 @@ xml_sax_wb (XMLSaxParseState *state, CHAR const **attrs)
 			};
 			int i;
 			for (i = 0 ; GnumericVersions [i].id != NULL ; ++i )
-				if (strcmp (attrs[0], GnumericVersions [i].id) == 0) {
+				if (strcmp (attrs[1], GnumericVersions [i].id) == 0) {
 					if (state->version != GNUM_XML_UNKNOWN)
 						xml_sax_warning (state, "Multiple version specifications.  Assuming %d",
 								state->version);
-					else
+					else {
 						state->version = GnumericVersions [i].version;
+						break;
+					}
 				}
 		} else
 			xml_sax_unknown_attr (state, attrs, "Workbook");
 }
 
+static void
+xml_sax_wb_sheetname (XMLSaxParseState *state)
+{
+	char const * content = state->content->str;
+	Sheet *sheet = sheet_new (state->wb, content);
+	workbook_sheet_attach (state->wb, sheet, NULL);
+}
+
 static void
 xml_sax_wb_view (XMLSaxParseState *state, CHAR const **attrs)
 {
@@ -666,8 +704,20 @@ xml_sax_sheet_name (XMLSaxParseState *state)
 	char const * content = state->content->str;
 	g_return_if_fail (state->sheet == NULL);
 
-	state->sheet = sheet_new (state->wb, content);
+	/*
+	 * FIXME: Pull this out at some point, so we don't
+	 * have to support < GNUM_XML_V7 anymore
+	 */
+	if (state->version >= GNUM_XML_V7) {
+		state->sheet = workbook_sheet_by_name (state->wb, content);
 
+		if (!state->sheet)
+			xml_sax_fatal_error (state, "SheetNameIndex reading failed");
+	} else {
+		state->sheet = sheet_new (state->wb, content);
+		workbook_sheet_attach (state->wb, state->sheet, NULL);
+	}
+	
 	if (state->display_formulas >= 0)
 		state->sheet->display_formulas = state->display_formulas;
 	if (state->hide_zero >= 0)
@@ -684,8 +734,6 @@ xml_sax_sheet_name (XMLSaxParseState *state)
 		state->sheet->outline_symbols_below = state->outline_symbols_below;
 	if (state->outline_symbols_right >= 0)
 		state->sheet->outline_symbols_right = state->outline_symbols_right;
-
-	workbook_sheet_attach (state->wb, state->sheet, NULL);
 }
 
 static void
@@ -1111,11 +1159,9 @@ xml_cell_set_array_expr (Cell *cell, char const *text,
 
 	g_return_if_fail (expr != NULL);
 	cell_set_array_formula (cell->base.sheet,
-				cell->pos.row,
-				cell->pos.col,
-				cell->pos.row + rows-1,
-				cell->pos.col + cols-1,
-				expr, TRUE);
+				cell->pos.col, cell->pos.row,
+				cell->pos.col + cols-1, cell->pos.row + rows-1,
+				expr);
 }
 
 /**
@@ -1147,13 +1193,13 @@ xml_not_used_old_array_spec (Cell *cell, char const *content)
 	rows = strtol (ptr = expr_end + 2, &end, 10);
 	if (end == ptr || *end != ',')
 		return TRUE;
-	cols = strtol (ptr = ++end, &end, 10);
+	cols = strtol (ptr = end + 1, &end, 10);
 	if (end == ptr || end[0] != ')' || end[1] != '[')
 		return TRUE;
-	row = strtol (ptr = (end += 2), &end, 10);
+	row = strtol (ptr = end + 2, &end, 10);
 	if (end == ptr || end[0] != ']' || end[1] != '[')
 		return TRUE;
-	col = strtol (ptr = (end += 2), &end, 10);
+	col = strtol (ptr = end + 2, &end, 10);
 	if (end == ptr || end[0] != ']' || end[1] != '\0')
 		return TRUE;
 
@@ -1268,6 +1314,78 @@ xml_sax_merge (XMLSaxParseState *state)
 static void
 xml_sax_object (XMLSaxParseState *state, CHAR const **attrs)
 {
+	
+}
+
+static void
+xml_sax_finish_parse_wb_names_name (XMLSaxParseState *state)
+{
+	g_return_if_fail (state->name.name != NULL);
+	g_return_if_fail (state->name.value != NULL);
+	
+	if (state->version >= GNUM_XML_V7) {
+		ParseError  perr;
+		
+		if (!expr_name_create (state->wb, NULL, state->name.name,
+				       state->name.value, &perr))
+			g_warning (perr.message);
+		parse_error_free (&perr);
+	} else {
+		/*
+		 * We can't do this for versions < V7. The problem
+		 * is that we really need the SheetNameIndex for this
+		 * to function correctly.
+		 * FIXME: We should fallback to the xml DOM parser
+		 * when this fails.
+		 */
+		g_warning ("Can't process named expression '%s'. Ignoring!", state->name.name);
+	}
+
+	g_free (state->name.name);
+	g_free (state->name.value);
+	state->name.name = NULL;
+	state->name.value = NULL;
+}
+
+static void
+xml_sax_finish_parse_sheet_names_name (XMLSaxParseState *state)
+{
+	ParseError  perr;
+	
+	g_return_if_fail (state->name.name != NULL);
+	g_return_if_fail (state->name.value != NULL);
+
+	if (!expr_name_create (NULL, state->sheet, state->name.name,
+			       state->name.value, &perr))
+		g_warning (perr.message);
+	parse_error_free (&perr);
+			  
+	g_free (state->name.name);
+	g_free (state->name.value);
+	state->name.name = NULL;
+	state->name.value = NULL;
+}
+
+static void
+xml_sax_name (XMLSaxParseState *state)
+{
+	char const * content = state->content->str;
+	int const len = state->content->len;
+
+	switch (state->state) {
+	case STATE_SHEET_NAMES_NAME_NAME:
+	case STATE_NAMES_NAME_NAME:
+		g_return_if_fail (state->name.name == NULL);
+		state->name.name = g_strndup (content, len);
+		break;
+	case STATE_SHEET_NAMES_NAME_VALUE:
+	case STATE_NAMES_NAME_VALUE:
+		g_return_if_fail (state->name.value == NULL);
+		state->name.value = g_strndup (content, len);
+		break;
+	default:
+		return;
+	}
 }
 
 /****************************************************************************/
@@ -1295,7 +1413,7 @@ xml_sax_unknown_state (XMLSaxParseState *state, CHAR const *name)
  * We parse and do some limited validation of the XML file, if this
  * passes, then we return TRUE
 gboolean
-xmlSax_file_probe (FileOpener const *fo, const gchar *file_name)
+xml_sax_file_probe (GnumFileOpener const *fo, const gchar *file_name, FileProbeLevel pl)
 {
 	return TRUE;
 }
@@ -1315,13 +1433,13 @@ xml_sax_start_element (XMLSaxParseState *state, CHAR const *name, CHAR const **a
 	case STATE_WB :
 		if (xml_sax_switch_state (state, name, STATE_WB_ATTRIBUTES)) {
 		} else if (xml_sax_switch_state (state, name, STATE_WB_SUMMARY)) {
+		} else if (xml_sax_switch_state (state, name, STATE_WB_SHEETNAME_INDEX)) {
 		} else if (xml_sax_switch_state (state, name, STATE_WB_GEOMETRY)) {
 			xml_sax_wb_view (state, attrs);
 		} else if (xml_sax_switch_state (state, name, STATE_WB_SHEETS)) {
 		} else if (xml_sax_switch_state (state, name, STATE_WB_VIEW)) {
 			xml_sax_wb_view (state, attrs);
 		} else if (xml_sax_switch_state (state, name, STATE_NAMES)) {
-			/* TODO : parse these */
 		} else
 			xml_sax_unknown_state (state, name);
 		break;
@@ -1354,6 +1472,12 @@ xml_sax_start_element (XMLSaxParseState *state, CHAR const *name, CHAR const **a
 			xml_sax_unknown_state (state, name);
 		break;
 
+	case STATE_WB_SHEETNAME_INDEX :
+		if (xml_sax_switch_state (state, name, STATE_WB_SHEETNAME)) {
+		} else
+			xml_sax_unknown_state (state, name);
+		break;
+		
 	case STATE_WB_SHEETS :
 		if (xml_sax_switch_state (state, name, STATE_SHEET)) {
 			xml_sax_sheet_start (state, attrs);
@@ -1366,6 +1490,7 @@ xml_sax_start_element (XMLSaxParseState *state, CHAR const *name, CHAR const **a
 		} else if (xml_sax_switch_state (state, name, STATE_SHEET_MAXCOL)) {
 		} else if (xml_sax_switch_state (state, name, STATE_SHEET_MAXROW)) {
 		} else if (xml_sax_switch_state (state, name, STATE_SHEET_ZOOM)) {
+		} else if (xml_sax_switch_state (state, name, STATE_SHEET_NAMES)) {
 		} else if (xml_sax_switch_state (state, name, STATE_SHEET_PRINTINFO)) {
 		} else if (xml_sax_switch_state (state, name, STATE_SHEET_STYLES)) {
 		} else if (xml_sax_switch_state (state, name, STATE_SHEET_COLS)) {
@@ -1381,7 +1506,19 @@ xml_sax_start_element (XMLSaxParseState *state, CHAR const *name, CHAR const **a
 		} else
 			xml_sax_unknown_state (state, name);
 		break;
-
+		
+	case STATE_SHEET_NAMES:
+		if (xml_sax_switch_state (state, name, STATE_SHEET_NAMES_NAME)) {
+		} else
+			xml_sax_unknown_state (state, name);
+		break;
+	case STATE_SHEET_NAMES_NAME:
+		if (xml_sax_switch_state (state, name, STATE_SHEET_NAMES_NAME_NAME)) {
+		} else if (xml_sax_switch_state (state, name, STATE_SHEET_NAMES_NAME_VALUE)) {
+		} else
+			xml_sax_unknown_state (state, name);
+		break;
+		
 	case STATE_SHEET_MERGED_REGION :
 		if (!xml_sax_switch_state (state, name, STATE_SHEET_MERGE))
 			xml_sax_unknown_state (state, name);
@@ -1505,6 +1642,18 @@ xml_sax_start_element (XMLSaxParseState *state, CHAR const *name, CHAR const **a
 		} else
 			xml_sax_unknown_state (state, name);
 		break;
+		
+	case STATE_NAMES:
+		if (xml_sax_switch_state (state, name, STATE_NAMES_NAME)) {
+		} else
+			xml_sax_unknown_state (state, name);
+		break;
+	case STATE_NAMES_NAME:
+		if (xml_sax_switch_state (state, name, STATE_NAMES_NAME_NAME)) {
+		} else if (xml_sax_switch_state (state, name, STATE_NAMES_NAME_VALUE)) {
+		} else
+			xml_sax_unknown_state (state, name);
+		break;
 
 	default :
 		break;
@@ -1532,7 +1681,7 @@ xml_sax_end_element (XMLSaxParseState *state, const CHAR *name)
 		break;
 
 	case STATE_WB_ATTRIBUTES :
-		wb_view_set_attributev (state->wb_view, state->attributes);
+		wb_view_set_attribute_list (state->wb_view, state->attributes);
 		xml_sax_free_arg_list (state->attributes);
 		state->attributes = NULL;
 		break;
@@ -1563,6 +1712,11 @@ xml_sax_end_element (XMLSaxParseState *state, const CHAR *name)
 		g_string_truncate (state->content, 0);
 		break;
 
+	case STATE_WB_SHEETNAME :
+		xml_sax_wb_sheetname (state);
+		g_string_truncate (state->content, 0);
+		break;
+		
 	case STATE_SHEET_NAME :
 		xml_sax_sheet_name (state);
 		g_string_truncate (state->content, 0);
@@ -1572,7 +1726,16 @@ xml_sax_end_element (XMLSaxParseState *state, const CHAR *name)
 		xml_sax_sheet_zoom (state);
 		g_string_truncate (state->content, 0);
 		break;
-
+		
+	case STATE_SHEET_NAMES_NAME :
+		xml_sax_finish_parse_sheet_names_name (state);
+		break;
+	case STATE_SHEET_NAMES_NAME_NAME :
+	case STATE_SHEET_NAMES_NAME_VALUE :
+		xml_sax_name (state);
+		g_string_truncate (state->content, 0);
+		break;
+		
 	case STATE_PRINT_MARGIN_TOP :
 	case STATE_PRINT_MARGIN_BOTTOM :
 	case STATE_PRINT_MARGIN_LEFT :
@@ -1600,6 +1763,15 @@ xml_sax_end_element (XMLSaxParseState *state, const CHAR *name)
 		xml_sax_merge (state);
 		g_string_truncate (state->content, 0);
 		break;
+		
+	case STATE_NAMES_NAME :
+		xml_sax_finish_parse_wb_names_name (state);
+		break;
+	case STATE_NAMES_NAME_NAME :
+	case STATE_NAMES_NAME_VALUE :
+		xml_sax_name (state);
+		g_string_truncate (state->content, 0);
+		break;
 	default :
 		break;
 	};
@@ -1619,8 +1791,11 @@ xml_sax_characters (XMLSaxParseState *state, const CHAR *chars, int len)
 	case STATE_WB_SUMMARY_ITEM_NAME :
 	case STATE_WB_SUMMARY_ITEM_VALUE_INT :
 	case STATE_WB_SUMMARY_ITEM_VALUE_STR :
+	case STATE_WB_SHEETNAME :
 	case STATE_SHEET_NAME :
 	case STATE_SHEET_ZOOM :
+	case STATE_SHEET_NAMES_NAME_NAME :
+	case STATE_SHEET_NAMES_NAME_VALUE :
 	case STATE_PRINT_MARGIN_TOP :
 	case STATE_PRINT_MARGIN_BOTTOM :
 	case STATE_PRINT_MARGIN_LEFT :
@@ -1634,6 +1809,8 @@ xml_sax_characters (XMLSaxParseState *state, const CHAR *chars, int len)
 	case STATE_STYLE_FONT :
 	case STATE_CELL_CONTENT :
 	case STATE_SHEET_MERGE :
+	case STATE_NAMES_NAME_NAME :
+	case STATE_NAMES_NAME_VALUE :
 		while (len-- > 0)
 			g_string_append_c (state->content, *chars++);
 
@@ -1664,6 +1841,8 @@ xml_sax_start_document (XMLSaxParseState *state)
 	state->attribute.type = -1;
 	state->attributes = NULL;
 
+	state->name.name = state->name.value = NULL;
+
 	state->style_range_init = FALSE;
 	state->style = NULL;
 
@@ -1714,7 +1893,7 @@ static xmlSAXHandler xmlSaxSAXParser = {
 };
 
 void
-xml_sax_file_open (FileOpener const *fo, IOContext *io_context,
+xml_sax_file_open (GnumFileOpener const *fo, IOContext *io_context,
 		   WorkbookView *wb_view, char const *filename)
 {
 	xmlParserCtxtPtr ctxt;
@@ -1747,6 +1926,8 @@ xml_sax_file_open (FileOpener const *fo, IOContext *io_context,
 		gnumeric_io_error_info_set (io_context,
 		                            error_info_new_str (
 		                            _("XML document not well formed!")));
+	else
+		workbook_queue_all_recalc (state.wb);
 
 	ctxt->sax = NULL;
 	xmlFreeParserCtxt (ctxt);
