@@ -155,19 +155,6 @@ colrow_index_compare (ColRowIndex const * a, ColRowIndex const * b)
 	return a->first - b->first;
 }
 
-static void
-colrow_string_build (GString *str, int index, gboolean const is_cols)
-{	
-	if (is_cols)
-		g_string_append (str, col_name (index));
-	else {
-		char *s = g_strdup_printf ("%d", index + 1);
-			
-		g_string_append (str, s);
-		g_free (s);
-	}
-}
-
 /*
  * colrow_index_list_to_string: Convert an index list into a string.
  *                              The result must be freed by the caller.
@@ -189,13 +176,14 @@ colrow_index_list_to_string (ColRowIndexList *list, gboolean const is_cols, gboo
 	result = g_string_new ("");
 	for (ptr = list; ptr != NULL; ptr = ptr->next) {
 		ColRowIndex *index = ptr->data;
-		
-		colrow_string_build (result, index->first, is_cols);
-		if (index->last != index->first) {
-			g_string_append (result, "-");
-			colrow_string_build (result, index->last, is_cols);
+
+		if (is_cols)
+			g_string_append (result, cols_name (index->first, index->last));
+		else
+			g_string_append (result, rows_name (index->first, index->last));
+			
+		if (index->last != index->first)
 			single = FALSE;
-		}
 		
 		if (ptr->next) {
 			g_string_append (result, ", ");
@@ -710,6 +698,30 @@ colrow_set_visibility_list (Sheet *sheet, gboolean const is_cols,
 	sheet_redraw_headers (sheet, TRUE, TRUE, NULL);
 }
 
+/**
+ * colrow_set_outline :
+ *
+ * Sets the outline level for a col/row.
+ * Returns the new outline level of @index.
+ */
+int
+colrow_set_outline (ColRowInfo *cri, gboolean is_cols,
+		    int outline_level, gboolean relative,
+		    gboolean is_collapsed)
+{
+	int newlevel = relative ? cri->outline_level + outline_level : outline_level;
+
+	cri->is_collapsed = (is_collapsed != 0);  /* needed for XL plugin */
+	g_return_val_if_fail (newlevel >= 0, 0);
+	
+	cri->outline_level = newlevel;	
+#if 0
+	printf ("%d = %d %d\n", index+1, outline_level, is_collapsed);
+#endif
+
+	return cri->outline_level;
+}
+
 /**
  * colrow_adjust_outline_dir 
  * @colrows :
