@@ -186,12 +186,13 @@ typedef struct {
 	gboolean    has_fraction;
         char        restriction_type;
         int         restriction_value;
+	StyleColor *color;
 } StyleFormatEntry;
 
 struct _StyleFormat {
 	int    ref_count;
 	char  *format;
-        GList *entries;  /* Of type StyleFormatEntry. */
+        GSList *entries;  /* Of type StyleFormatEntry. */
 };
 
 /*
@@ -395,158 +396,17 @@ append_second_elapsed (GString *string, int n, const struct tm *time_split, int
 	return;
 }
 
-#if 0
-/*
- * Parses the day part field at the beginning of the format.  Returns
- * the number of characters used.
- */
-static int
-append_half (GString *string, const guchar *format, const struct tm *time_split)
+static StyleFormatEntry *
+format_entry_ctor (void)
 {
-	if (time_split->tm_hour <= 11){
-		if (tolower (format [0]) == 'a' || tolower (format [0]) == 'p')
-			g_string_append_c (string, 'a');
-		else
-			g_string_append_c (string, 'A');
-	}
-	else {
-		if (tolower (format [0]) == 'a' || tolower (format [0]) == 'p')
-			g_string_append_c (string, 'p');
-		else
-			g_string_append_c (string, 'P');
-	}
+	StyleFormatEntry *entry;
 
-	if (tolower (format [1]) == 'm'){
-		g_string_append_c (string, format [1]);
-		return 2;
-	} else
-		return 1;
-}
-#endif
-
-/*
- * Since the Excel formating codes contain a number of ambiguities,
- * this routine does some analysis on the format first.
- */
-static void
-pre_parse_format (StyleFormatEntry *style)
-{
-	const unsigned char *format;
-
-	style->want_am_pm = style->has_fraction = FALSE;
-	for (format = style->format; *format; format++){
-		switch (*format){
-		case '"':
-			format++;
-			while (*format && *format != '"')
-				format++;
-
-			if (!*format)
-				return;
-			break;
-
-		case '\\':
-			if (*(format+1))
-				format++;
-			else
-				return;
-			break;
-
-		case '/':
-			if (format [1] == '?' || isdigit (format[1])) {
-				style->has_fraction = TRUE;
-				format++;
-			}
-			break;
-		case 'a':
-		case 'p':
-		case 'A':
-		case 'P':
-			if (tolower (format[1]) == 'm')
-				style->want_am_pm = TRUE;
-			break;
-		default :
-			break;
-		}
-	}
-}
-
-typedef struct
-{
-	int decimal;
-	int timeformat;
-	int hasnumbers;
-} xformat_info;
-
-/*
- * This routine should always return, it cannot fail, in the worst
- * case it should just downgrade to simplistic formatting
- */
-static void
-format_compile (StyleFormat *format)
-{
-	GString *string = g_string_new ("");
-	int i;
-	int which = 0;
-	int length = strlen (format->format);
-	StyleFormatEntry standard_entries[4];
-	StyleFormatEntry *temp;
-
-	/* g_string_maybe_expand (string, length); */
-
-	for (i = 0; i < length; i++){
-
-		switch (format->format[i]){
-		case ';':
-			if (which < 4) {
-				standard_entries [which].restriction_type = '*';
-				standard_entries [which].format =
-					g_strndup (string->str, string->len + 1);
-				which++;
-			}
-			string = g_string_truncate (string, 0);
-			break;
-		default:
-			string = g_string_append_c (string, format->format [i]);
-			break;
-		}
-	}
-
-	if (which < 4) {
-		standard_entries[which].restriction_type = '*';
-		standard_entries[which].format =
-			g_strndup (string->str, string->len + 1);
-		which++;
-	}
-
-	/* Set up restriction types. */
-	standard_entries[1].restriction_type = '<';
-	standard_entries[1].restriction_value = 0;
-	switch (which){
-
-	case 4:
-		standard_entries[3].restriction_type = '@';
-		/* Fall through. */
-	case 3:
-		standard_entries[2].restriction_type = '=';
-		standard_entries[2].restriction_value = 0;
-		standard_entries[0].restriction_type = '>';
-		standard_entries[0].restriction_value = 0;
-		break;
-	case 2:
-		standard_entries[0].restriction_type = '.';  /* . >= */
-		standard_entries[0].restriction_value = 0;
-		break;
-	}
-
-	for (i = 0; i < which; i++){
-		temp = g_new (StyleFormatEntry, 1);
-		*temp = standard_entries[i];
-		pre_parse_format (temp);
-		format->entries = g_list_append (format->entries, temp);
-	}
-
-	g_string_free (string, TRUE);
+	entry = g_new (StyleFormatEntry, 1);
+	entry->restriction_type = '*';
+	entry->restriction_value = 0.;
+	entry->want_am_pm = entry->has_fraction = FALSE;
+	entry->color = NULL;
+	return entry;
 }
 
 /**
@@ -559,10 +419,172 @@ static void
 format_entry_dtor (gpointer data, gpointer user_data)
 {
 	StyleFormatEntry *entry = data;
+	if (entry->color != NULL) {
+		style_color_unref (entry->color);
+		entry->color = NULL;
+	}
 	g_free ((char *)entry->format);
 	g_free (entry);
 }
 
+static void
+format_entry_set_fmt (StyleFormatEntry *entry,
+		      unsigned char const *begin,
+		      unsigned char const *end)
+{
+	/* empty formats are General if there is a color, or a condition */
+	entry->format = (begin != NULL && end != begin)
+		? g_strndup (begin, end - begin)
+		: strdup ((entry->color != NULL ||
+			   entry->restriction_type != '*')
+			  ? "General" : "");
+}
+
+static StyleColor * lookup_color (const char *str, const char *end);
+
+/*
+ * Since the Excel formating codes contain a number of ambiguities, this
+ * routine does some analysis on the format first.  This routine should always
+ * return, it cannot fail, in the worst case it should just downgrade to
+ * simplistic formatting
+ */
+static void
+format_compile (StyleFormat *format)
+{
+	unsigned char const *fmt, *real_start = NULL;
+	StyleFormatEntry *entry = format_entry_ctor ();
+	int num_entries = 1, counter = 0;
+	GSList *ptr;
+
+	for (fmt = format->format; *fmt ; fmt++) {
+		if (NULL == real_start && '[' != *fmt)
+			real_start = fmt;
+
+		switch (*fmt) {
+		case '[': {
+			unsigned char const *begin = fmt + 1;
+			unsigned char const *end = begin;
+
+			/* find end checking for escapes but not quotes ?? */
+			for (; end[0] != ']' && end[1] != '\0' ; ++end)
+				if (*end == '\\')
+					end++;
+
+			/* Check for conditional */
+			if (*begin == '<') {
+				if (begin [1] == '=') {
+					entry->restriction_type = ',';
+					begin += 2;
+				} else if (begin [1] == '>') {
+					entry->restriction_type = '+';
+					begin += 2;
+				} else {
+					entry->restriction_type = '<';
+					begin++;
+				}
+			} else if (*begin == '>') {
+				if (begin [1] == '=') {
+					entry->restriction_type = '.';
+					begin += 2;
+				} else {
+					entry->restriction_type = '>';
+					begin++;
+				}
+			} else if (*begin == '=') {
+				entry->restriction_type = '=';
+			} else if (*begin != '$' && entry->color == NULL) {
+				entry->color = lookup_color (begin, end);
+				/* Only the first colour counts */
+				if (NULL == entry->color) {
+					if (NULL == real_start)
+						real_start = fmt;
+					continue;
+				}
+				fmt = end;
+				continue;
+			} else {
+				if (NULL == real_start)
+					real_start = fmt;
+				continue;
+			}
+			fmt = end;
+
+			/* fall back on 0 for errors */
+			errno = 0;
+			entry->restriction_value = strtod (begin, (char **)&end);
+			if (errno == ERANGE || begin == end)
+				entry->restriction_value = 0.;
+			break;
+		}
+
+		case '\\' :
+			if (fmt[1] != '\0')
+				fmt++; /* skip escaped characters */
+			break;
+
+		case '\'' :
+		case '\"' : {
+			/* skip quoted strings */
+			char const match = *fmt;
+			for (; fmt[0] != match && fmt[1] != '\0'; fmt++)
+				if (*fmt == '\\')
+					fmt++;
+			break;
+		}
+
+		case '/':
+			if (fmt [1] == '?' || isdigit (fmt[1])) {
+				entry->has_fraction = TRUE;
+				fmt++;
+			}
+			break;
+
+		case 'a': case 'A':
+		case 'p': case 'P':
+			if (tolower (fmt [1]) == 'm')
+				entry->want_am_pm = TRUE;
+			break;
+
+		case ';':
+			format_entry_set_fmt (entry, real_start, fmt);
+			format->entries = g_slist_append (format->entries, entry);
+			num_entries++;
+
+			entry = format_entry_ctor ();
+			real_start = NULL;
+			break;
+
+		default :
+			break;
+		}
+	}
+
+	format_entry_set_fmt (entry, real_start, fmt);
+	format->entries = g_slist_append (format->entries, entry);
+
+	for (ptr = format->entries; ptr && counter++ < 4 ; ptr = ptr->next) {
+		StyleFormatEntry *entry = ptr->data;
+
+		if (entry->restriction_type == '*') {
+			entry->restriction_value = 0.;
+
+			switch (counter) {
+			/* single conditions should remain catch alls */
+			case 1 : if (num_entries > 1)
+					 entry->restriction_type =
+						 (num_entries > 2) ? '>' : '.';
+				 break;
+			/* Once there is more than one condition no catchall */
+			case 2 : entry->restriction_type = '<'; break;
+			case 3 : entry->restriction_type = '='; break;
+			case 4 : entry->restriction_type = '@'; break;
+			default :
+				 break;
+			}
+		}
+	}
+}
+
 /*
  * This routine is invoked when the last user of the
  * format is gone (ie, refcount has reached zero) just
@@ -573,8 +595,8 @@ format_entry_dtor (gpointer data, gpointer user_data)
 void
 format_destroy (StyleFormat *format)
 {
-	g_list_foreach (format->entries, &format_entry_dtor, NULL);
-	g_list_free (format->entries);
+	g_slist_foreach (format->entries, &format_entry_dtor, NULL);
+	g_slist_free (format->entries);
 	format->entries = NULL;
 }
 
@@ -619,7 +641,7 @@ format_color_shutdown (void)
 }
 
 static struct FormatColor const *
-lookup_color_by_name (const char *str, const char *end,
+lookup_color_by_name (unsigned char const *str, unsigned char const *end,
 		      gboolean const translate)
 {
 	int i, len;
@@ -662,8 +684,10 @@ typedef struct {
 	gboolean has_fraction;
 } format_info_t;
 
-static GString *
-render_number (gdouble number,
+static double beyond_precision;
+static void
+render_number (GString *result,
+	       gdouble number,
 	       int left_req,
 	       int right_req,
 	       int left_spaces,
@@ -672,107 +696,98 @@ render_number (gdouble number,
 	       char const *show_decimal,
 	       format_info_t const *info)
 {
-	GString *number_string = g_string_new ("");
-	gint zero_count;
-	gdouble temp;
-	int group = 0;
 	char thousands_sep;
-	static gdouble beyond_precision = 0;
-
-	if (!beyond_precision)
-		beyond_precision = gpow10 (DBL_DIG + 1);
-
-	if (!info->has_fraction && right_allowed >= 0) {
-		/* Change "rounding" into "truncating".  */
-		gdouble delta = 0.5;
-		int i;
-		for (i = 0; i < right_allowed; i++)
-			delta /= 10.0;
-		/* Note, that we assume number >= 0 here.  */
-		number += delta;
-	}
+	unsigned char num_buf [(DBL_MANT_DIG + DBL_MAX_EXP)*2 + 1];
+	unsigned char *num = num_buf + sizeof(num_buf) - 1;
+	double frac_part, int_part;
+	int group, zero_count, digit_count = 0;
 
 	thousands_sep = format_get_thousand ();
 
-	for (temp = number; temp >= 1.0; temp /= 10.0) {
-		int digit;
+	if (right_allowed >= 0 && !info->has_fraction) {
+		/* Change "rounding" into "truncating".   */
+		/* Note, that we assume number >= 0 here. */
+		gdouble delta = 5 * gpow10 (-right_allowed-1);
+		number += delta;
+	}
+	frac_part = modf (gnumeric_add_epsilon (number), &int_part);
 
-		if (info->comma_separator_seen) {
-			group++;
-			if (group == 4){
-				group = 1;
-				g_string_prepend_c (number_string, thousands_sep);
-			}
+	*num = '\0';
+	group = (info->comma_separator_seen) ? 3 : -1;
+	for (; int_part > beyond_precision ; int_part /= 10., digit_count++) {
+		if (group-- == 0) {
+			group = 2;
+			*(--num) = thousands_sep;
 		}
-
-		if (temp > beyond_precision)
-			digit = 0;
-		else {
-			double r = floor (temp);
-			digit = r - floor (r / 10) * 10;
-		}
-		g_string_prepend_c (number_string, digit + '0');
-		if (left_req > 0)
-			left_req--;
-		if (left_spaces > 0)
-			left_spaces--;
+		*(--num) = '0';
 	}
 
-	for (; left_req > 0; left_req--, left_spaces--)
-		g_string_prepend_c (number_string, '0');
+	for (; int_part >= 1. ; int_part /= 10., digit_count++) {
+		double r = floor (int_part);
+		int digit = r - floor (r / 10) * 10;
 
-	for (; left_spaces > 0; left_spaces--)
-		g_string_prepend_c (number_string, ' ');
+		if (group-- == 0) {
+			group = 2;
+			*(--num) = thousands_sep;
+		}
+		*(--num) = digit + '0';
+	}
 
+	/* TODO : What ifthe only visible digits are zeros ? -0.00 looks bad */
 	if (info->negative && !info->supress_minus)
-		g_string_prepend_c (number_string, '-');
+		g_string_append_c (result, '-');
+	if (left_req > digit_count) {
+		for (left_spaces -= left_req ; left_spaces-- > 0 ;)
+			g_string_append_c (result, ' ');
+		for (left_req -= digit_count ; left_req-- > 0 ;)
+			g_string_append_c (result, '0');
+	}
+
+	g_string_append (result, num);
 
 	if (info->decimal_separator_seen)
-		g_string_append_c (number_string, format_get_decimal ());
+		g_string_append_c (result, format_get_decimal ());
 	else
-		g_string_append (number_string, show_decimal);
+		g_string_append (result, show_decimal);
 
-	temp = number - floor (number);
-
-	for (; right_req > 0; right_req --, right_allowed --, right_spaces --) {
+	/* TODO : clip this a DBL_DIG */
+	/* TODO : What if is a fraction ? */
+	right_allowed -= right_req;
+	right_spaces  -= right_req;
+	while (right_req-- > 0) {
 		gint digit;
-		temp *= 10.0;
-		digit = (gint)temp;
-		temp -= digit;
-		g_string_append_c (number_string, digit + '0');
+		frac_part *= 10.0;
+		digit = (gint)frac_part;
+		frac_part -= digit;
+		g_string_append_c (result, digit + '0');
 	}
 
 	zero_count = 0;
 
-	for (; right_allowed > 0; right_allowed --) {
+	while (right_allowed-- > 0) {
 		gint digit;
-		temp *= 10.0;
-		digit = (gint)temp;
-		temp -= digit;
+		frac_part *= 10.0;
+		digit = (gint)frac_part;
+		frac_part -= digit;
 
-		if (digit == 0)
-			zero_count ++;
-		else {
+		if (digit == 0) {
 			right_spaces -= zero_count + 1;
 			zero_count = 0;
-		}
+		} else
+			zero_count ++;
 
-		g_string_append_c (number_string, digit + '0');
+		g_string_append_c (result, digit + '0');
 	}
 
-	g_string_truncate (number_string, number_string->len - zero_count);
+	g_string_truncate (result, result->len - zero_count);
 
-	for (; right_spaces > 0; right_spaces--) {
-		g_string_append_c (number_string, ' ');
-	}
-
-	return number_string;
+	while (right_spaces-- > 0)
+		g_string_append_c (result, ' ');
 }
 
 static void
 do_render_number (gdouble number, format_info_t *info, GString *result)
 {
-	GString *number_str;
 	char decimal_point [2];
 
 	info->rendered = TRUE;
@@ -810,7 +825,7 @@ do_render_number (gdouble number, format_info_t *info, GString *result)
 		decimal_point);
 #endif
 
-	number_str = render_number (
+	render_number (result,
 		number,
 		info->left_req,
 		info->right_req,
@@ -819,9 +834,6 @@ do_render_number (gdouble number, format_info_t *info, GString *result)
 		info->right_allowed + info->right_optional,
 		decimal_point,
 		info);
-
-	g_string_append (result, number_str->str);
-	g_string_free (number_str, TRUE);
 }
 
 /*
@@ -1021,41 +1033,6 @@ format_add_decimal (StyleFormat const *fmt)
 
 /*********************************************************************/
 
-static void
-stern_brocot (float val, int max_denom, int *res_num, int *res_denom)
-{
-	int an = 0, ad = 1;
-	int bn = 1, bd = 1;
-	int n, d;
-	float sp, delta;
-
-	while ((d = ad + bd) <= max_denom) {
-		sp = 1e-5 * d;	/* Quick and dirty,  do adaptive later */
-		n = an + bn;
-		delta = val * d - n;
-		if (delta > sp)
-		{
-			an = n;
-			ad = d;
-		} else if (delta < -sp)
-		{
-			bn = n;
-			bd = d;
-		} else {
-			*res_num = n;
-			*res_denom = d;
-			return;
-		}
-	}
-	if (bd > max_denom || fabs (val * ad - an) < fabs (val * bd - bn)) {
-		*res_num = an;
-		*res_denom = ad;
-	} else {
-		*res_num = bn;
-		*res_denom = bd;
-	}
-}
-
 static gchar *
 format_number (gdouble number, int col_width, StyleFormatEntry const *entry)
 {
@@ -1184,8 +1161,8 @@ format_number (gdouble number, int col_width, StyleFormatEntry const *entry)
 
 		case '\\':
 			if (format[1] != '\0') {
-				/* TODO TODO TODO : Do we need any more chars here ?? */
-				if (format[1] == '-')
+				/* TODO : Other chars here ?? ('+' or ':') ? */
+				if (format[1] == '-' || format[1] == '(')
 					info.supress_minus = TRUE;
 				else if (can_render_number && !info.rendered)
 					do_render_number (number, &info, result);
@@ -1446,51 +1423,36 @@ format_number (gdouble number, int col_width, StyleFormatEntry const *entry)
 }
 
 static gboolean
-check_valid (const StyleFormatEntry *entry, const Value *value)
+style_format_condition (StyleFormatEntry const *entry, Value const *value)
 {
-	switch (value->type){
+	if (entry->restriction_type == '*')
+		return TRUE;
 
+	switch (value->type) {
+	case VALUE_ERROR:
 	case VALUE_STRING:
 		return entry->restriction_type == '@';
 
 	case VALUE_FLOAT:
-		switch (entry->restriction_type){
-
-		case '*':
-			return TRUE;
-		case '<':
-			return value->v_float.val < entry->restriction_value;
-		case '>':
-			return value->v_float.val > entry->restriction_value;
-		case '=':
-			return value->v_float.val == entry->restriction_value;
-		case ',':
-			return value->v_float.val <= entry->restriction_value;
-		case '.':
-			return value->v_float.val >= entry->restriction_value;
-		case '+':
-			return value->v_float.val != entry->restriction_value;
+		switch (entry->restriction_type) {
+		case '<': return value->v_float.val < entry->restriction_value;
+		case '>': return value->v_float.val > entry->restriction_value;
+		case '=': return value->v_float.val == entry->restriction_value;
+		case ',': return value->v_float.val <= entry->restriction_value;
+		case '.': return value->v_float.val >= entry->restriction_value;
+		case '+': return value->v_float.val != entry->restriction_value;
 		default:
 			return FALSE;
 		}
 
 	case VALUE_INTEGER:
-		switch (entry->restriction_type){
-
-		case '*':
-			return TRUE;
-		case '<':
-			return value->v_int.val < entry->restriction_value;
-		case '>':
-			return value->v_int.val > entry->restriction_value;
-		case '=':
-			return value->v_int.val == entry->restriction_value;
-		case ',':
-			return value->v_int.val <= entry->restriction_value;
-		case '.':
-			return value->v_int.val >= entry->restriction_value;
-		case '+':
-			return value->v_int.val != entry->restriction_value;
+		switch (entry->restriction_type) {
+		case '<': return value->v_int.val < entry->restriction_value;
+		case '>': return value->v_int.val > entry->restriction_value;
+		case '=': return value->v_int.val == entry->restriction_value;
+		case ',': return value->v_int.val <= entry->restriction_value;
+		case '.': return value->v_int.val >= entry->restriction_value;
+		case '+': return value->v_int.val != entry->restriction_value;
 		default:
 			return FALSE;
 		}
@@ -1598,77 +1560,62 @@ format_value (StyleFormat *format, const Value *value, StyleColor **color,
 	      float col_width)
 {
 	char *v = NULL;
-	StyleFormatEntry entry;
-	gboolean is_general = FALSE;
-	GList *list;
+	StyleFormatEntry const *entry = NULL; /* default to General */
+	GSList *list;
 
 	if (color)
 		*color = NULL;
 
 	g_return_val_if_fail (value != NULL, "<ERROR>");
 
-	if (format) {
-		/* get format */
-		for (list = format->entries; list; list = g_list_next (list))
-			if (check_valid (list->data, value))
-				break;
-
-		if (list)
-			entry = *(StyleFormatEntry *)(list->data);
-		else
-			entry.format = format->format;
-
-		/* Try to parse a color specification */
-		if (entry.format [0] == '['){
-			char *end = strchr (entry.format, ']');
-
-			if (end) {
-				char first_after_bracket = entry.format [1];
-
-				/*
-				 * Special [h*], [m*], [*s] is using for
-				 * and [$*] are for currencies.
-				 * measuring times, not for specifying colors.
-				 */
-				if (!(first_after_bracket == 'h' ||
-				      first_after_bracket == 's' ||
-				      first_after_bracket == 'm' ||
-				      first_after_bracket == '$')){
-					if (color)
-						*color = lookup_color (&entry.format [1], end);
-					entry.format = end+1;
-				}
-			}
-		}
-
-		/* Empty formats should be ignored */
-		if (entry.format [0] == '\0')
-			return g_strdup ("");
-
-		/* Formatting a value as a text returns the entered text */
-		if (strcmp (entry.format, "@") == 0) {
-			if (value->type == VALUE_STRING)
-				return g_strdup (value->v_str.val->str);
-
-			is_general = TRUE;
-		} else if (strcmp (entry.format, "General") == 0)
-			is_general = TRUE;
-	} else
-		is_general = TRUE;
-
-	/*
-	 * Use top left corner of an array result.
+	/* Use top left corner of an array result.
 	 * This wont work for ranges because we dont't have a location
 	 */
 	if (value->type == VALUE_ARRAY)
 		value = value_area_fetch_x_y (NULL, value, 0, 0);
 
-	switch (value->type){
+	if (format) {
+		/* get format */
+		for (list = format->entries; list; list = list->next)
+			if (style_format_condition (list->data, value))
+				break;
+
+		/* If nothing matches treat it as General */
+		if (NULL != list) {
+			entry = (StyleFormatEntry const *)(list->data);
+
+			/* Empty formats should be ignored */
+			if (entry->format [0] == '\0')
+				return g_strdup ("");
+
+			if (color && entry->color != NULL)
+				*color = style_color_ref (entry->color);
+
+			if (strcmp (entry->format, "@") == 0) {
+				/* FIXME : Formatting a value as a text returns
+				 * the entered text.  We need access to the
+				 * parse format */
+				entry = NULL;
+			} else if (strcmp (entry->format, "General") == 0)
+				entry = NULL;
+		}
+	}
+
+	switch (value->type) {
+	case VALUE_EMPTY:
+		return g_strdup ("");
+	case VALUE_BOOLEAN:
+		return g_strdup (value->v_bool.val ? _("TRUE"):_("FALSE"));
+	case VALUE_INTEGER:
+		if (entry == NULL)
+			return fmt_general_int (value->v_int.val, col_width);
+		v = format_number (value->v_int.val, (int)col_width, entry);
+		break;
 	case VALUE_FLOAT:
 		if (!FINITE (value->v_float.val))
 			return g_strdup (gnumeric_err_VALUE);
 
-		if (is_general) {
+		if (entry == NULL) {
 			gnum_float val = value->v_float.val;
 			if ((gnum_float)INT_MAX >= val && val >= (gnum_float)INT_MIN) {
 				double int_val = floor (value->v_float.val);
@@ -1677,34 +1624,17 @@ format_value (StyleFormat *format, const Value *value, StyleColor **color,
 			}
 			return fmt_general_float (val, col_width);
 		}
-		v = format_number (value->v_float.val, (int)col_width, &entry);
+		v = format_number (value->v_float.val, (int)col_width, entry);
 		break;
-
-	case VALUE_INTEGER:
-		if (is_general)
-			return fmt_general_int (value->v_int.val, col_width);
-		v = format_number (value->v_int.val, (int)col_width, &entry);
-		break;
-
-	case VALUE_BOOLEAN:
-		return g_strdup (value->v_bool.val ? _("TRUE"):_("FALSE"));
-
 	case VALUE_ERROR:
 		return g_strdup (value->v_err.mesg->str);
-
 	case VALUE_STRING:
 		return g_strdup (value->v_str.val->str);
-
 	case VALUE_CELLRANGE:
 		return g_strdup (gnumeric_err_VALUE);
-
-	case VALUE_ARRAY:
-		/* Array of arrays ?? */
+	case VALUE_ARRAY: /* Array of arrays ?? */
 		return g_strdup (_("ARRAY"));
 
-	case VALUE_EMPTY:
-		return g_strdup ("");
-
 	default:
 		return g_strdup ("Internal error");
 	}
@@ -1720,7 +1650,9 @@ void
 number_format_init (void)
 {
 	style_format_hash = g_hash_table_new (g_str_hash, g_str_equal);
+	beyond_precision = gpow10 (DBL_DIG + 1);
 }
+
 void
 number_format_shutdown (void)
 {
