@@ -8,18 +8,19 @@
 
  */
 #include <config.h>
-#include <math.h>
-#include <limits.h>
-#include "gnumeric.h"
+#include "func.h"
 #include "parse-util.h"
 #include "cell.h"
-#include "func.h"
 #include "goal-seek.h"
 #include "collect.h"
 #include "auto-format.h"
 #include "datetime.h"
 #include "str.h"
 
+#include <math.h>
+#include <limits.h>
+#include <string.h>
+
 /*
 
 Below are some of the functions that are used quite often in
@@ -591,15 +592,12 @@ static Value *
 gnumeric_accrint (FunctionEvalInfo *ei, Value **argv)
 {
 	gnum_float rate, a, d, par, freq, coefficient, x;
-	int     basis;
+	int basis;
 
 	rate = value_get_as_float (argv[3]);
 	par = value_get_as_float (argv[4]);
 	freq = value_get_as_float (argv[5]);
-	if (argv[6] == NULL)
-	        basis = 0;
-	else
-	        basis = value_get_as_int (argv[6]);
+	basis = argv[6] ? value_get_as_int (argv[6]) : 0;
 
 	a = days_monthly_basis (argv[0], argv[2], basis);
 	d = annual_year_basis (argv[0], basis);
@@ -649,7 +647,7 @@ static Value *
 gnumeric_accrintm (FunctionEvalInfo *ei, Value **argv)
 {
 	gnum_float rate, a, d, par;
-	int     basis;
+	int basis;
 
 	rate = value_get_as_float (argv[2]);
 	par = argv[3] ? value_get_as_float (argv[3]) : 1000;
@@ -708,7 +706,7 @@ static Value *
 gnumeric_intrate (FunctionEvalInfo *ei, Value **argv)
 {
 	gnum_float investment, redemption, a, d;
-	int     basis;
+	int basis;
 
 	investment = value_get_as_float (argv[2]);
 	redemption = value_get_as_float (argv[3]);
@@ -757,7 +755,7 @@ static Value *
 gnumeric_received (FunctionEvalInfo *ei, Value **argv)
 {
 	gnum_float investment, discount, a, d, n;
-	int     basis;
+	int basis;
 
 	investment = value_get_as_float (argv[2]);
 	discount = value_get_as_float (argv[3]);
@@ -811,7 +809,7 @@ static Value *
 gnumeric_pricedisc (FunctionEvalInfo *ei, Value **argv)
 {
 	gnum_float discount, redemption, a, d;
-	int     basis;
+	int basis;
 
 	discount = value_get_as_float (argv[2]);
 	redemption = value_get_as_float (argv[3]);
@@ -858,7 +856,7 @@ static Value *
 gnumeric_pricemat (FunctionEvalInfo *ei, Value **argv)
 {
 	gnum_float discount, yield, a, b, dsm, dim, n;
-	int     basis;
+	int basis;
 
 	discount = value_get_as_float (argv[3]);
 	yield = value_get_as_float (argv[4]);
@@ -912,7 +910,7 @@ static Value *
 gnumeric_disc (FunctionEvalInfo *ei, Value **argv)
 {
 	gnum_float par, redemption, dsm, b;
-	int     basis;
+	int basis;
 
 	par = value_get_as_float (argv[2]);
 	redemption = value_get_as_float (argv[3]);
@@ -970,10 +968,10 @@ gnumeric_effect (FunctionEvalInfo *ei, Value **argv)
 	nper = value_get_as_int (argv[1]);
 
 	/* Rate or number of periods cannot be negative */
-	if ((rate < 0) || (nper <= 0))
+	if (rate < 0 || nper <= 0)
 		return value_new_error (ei->pos, _("effect - domain error"));
 
-        return value_new_float ( pow ( (1 + rate/nper) , nper) - 1 );
+        return value_new_float (pow ((1 + rate / nper), nper) - 1);
 }
 
 /***************************************************************************/
@@ -1011,7 +1009,7 @@ gnumeric_nominal (FunctionEvalInfo *ei, Value **argv)
 	if (rate < 0 || nper <= 0)
 		return value_new_error (ei->pos, _("nominal - domain error"));
 
-        return value_new_float ( nper * ( pow ( 1 + rate, 1.0/nper ) - 1 ) );
+        return value_new_float (nper * (pow (1 + rate, 1.0 / nper ) - 1 ));
 
 }
 
@@ -1034,7 +1032,7 @@ static Value *
 gnumeric_ispmt (FunctionEvalInfo *ei, Value **argv)
 {
 	gnum_float rate, pv, tmp;
-	int     nper, per;
+	int nper, per;
 
 	rate = value_get_as_float (argv[0]);
 	per = value_get_as_int (argv[1]);
@@ -1044,7 +1042,7 @@ gnumeric_ispmt (FunctionEvalInfo *ei, Value **argv)
 	if (per < 1 || per > nper)
                 return value_new_error (ei->pos, gnumeric_err_NUM);
 
-	tmp = - pv * rate;
+	tmp = -pv * rate;
 
 	return value_new_float (tmp - (tmp / nper * per));
 }
@@ -1074,7 +1072,7 @@ gnumeric_db (FunctionEvalInfo *ei, Value **argv)
 	gnum_float rate;
 	gnum_float cost, salvage, life, period, month;
 	gnum_float total;
-	int     i;
+	int i;
 
 	cost = value_get_as_float (argv[0]);
 	salvage = value_get_as_float (argv[1]);
@@ -1082,12 +1080,13 @@ gnumeric_db (FunctionEvalInfo *ei, Value **argv)
 	period = value_get_as_float (argv[3]);
 	month = argv[4] ? value_get_as_float (argv[4]) : 12;
 
-	if (cost == 0 || life <= 0)
+	/* The third disjunct is a bit of a guess -- MW.  */
+	if (cost == 0 || life <= 0 || salvage / cost < 0)
 	        return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	rate = 1 - pow ((salvage / cost), (1 / life));
 	rate *= 1000;
-	rate = floor(rate+0.5) / 1000;
+	rate = floor (rate+0.5) / 1000;
 
 	total = cost * rate * month / 12;
 
@@ -1128,7 +1127,7 @@ gnumeric_ddb (FunctionEvalInfo *ei, Value **argv)
 {
 	gnum_float cost, salvage, life, period, factor;
 	gnum_float total;
-	int     i;
+	int i;
 
 	cost = value_get_as_float (argv[0]);
 	salvage = value_get_as_float (argv[1]);
@@ -1280,7 +1279,7 @@ static Value *
 gnumeric_dollarde (FunctionEvalInfo *ei, Value **argv)
 {
         gnum_float fractional_dollar;
-	int     fraction, n, tmp;
+	int fraction, n, tmp;
 	gnum_float floored, rest;
 
 	fractional_dollar = value_get_as_float (argv[0]);
@@ -1291,13 +1290,13 @@ gnumeric_dollarde (FunctionEvalInfo *ei, Value **argv)
 
 	tmp = fraction;
 	/* Count digits in fraction */
-	for (n=0; tmp; n++)
+	for (n = 0; tmp; n++)
 	        tmp /= 10;
 
 	floored = floor (fractional_dollar);
 	rest = fractional_dollar - floored;
 
-	return value_new_float (floored + ((gnum_float) rest * pow (10,n) /
+	return value_new_float (floored + ((gnum_float) rest * pow (10, n) /
 					   fraction));
 }
 
@@ -1322,7 +1321,7 @@ static Value *
 gnumeric_dollarfr (FunctionEvalInfo *ei, Value **argv)
 {
         gnum_float fractional_dollar;
-	int     fraction, n, tmp;
+	int fraction, n, tmp;
 	gnum_float floored, rest;
 
 	fractional_dollar = value_get_as_float (argv[0]);
@@ -1333,13 +1332,13 @@ gnumeric_dollarfr (FunctionEvalInfo *ei, Value **argv)
 
 	/* Count digits in fraction */
 	tmp = fraction;
-	for (n=0; tmp; n++)
+	for (n = 0; tmp; n++)
 	        tmp /= 10;
 
 	floored = floor (fractional_dollar);
 	rest = fractional_dollar - floored;
 
-	return value_new_float (floored + ((gnum_float) (rest*fraction) /
+	return value_new_float (floored + ((gnum_float) (rest * fraction) /
 					   pow (10, n)));
 }
 
@@ -1362,8 +1361,8 @@ gnumeric_mirr (FunctionEvalInfo *ei, Value **argv)
 {
 	gnum_float frate, rrate, npv_neg, npv_pos;
 	gnum_float *values = NULL, res;
-	Value   *result = NULL;
-	int     i, n;
+	Value *result = NULL;
+	int i, n;
 
 	frate = value_get_as_float (argv[1]);
 	rrate = value_get_as_float (argv[2]);
@@ -1604,16 +1603,10 @@ gnumeric_rate (FunctionEvalInfo *ei, Value **argv)
 #endif
 	goal_seek_initialise (&data);
 
-	/* Respect the sign of the guess.  */
-	if (rate0 >= 0) {
-		data.xmin = 0;
-		data.xmax = MIN (data.xmax,
-				 pow (DBL_MAX / 1e10, 1.0 / udata.nper) - 1);
-	} else {
-		data.xmin = MAX (data.xmin,
-				 -pow (DBL_MAX / 1e10, 1.0 / udata.nper) + 1);
-		data.xmax = 0;
-	}
+	data.xmin = MAX (data.xmin,
+			 -pow (DBL_MAX / 1e10, 1.0 / udata.nper) + 1);
+	data.xmax = MIN (data.xmax,
+			 pow (DBL_MAX / 1e10, 1.0 / udata.nper) - 1);
 
 	/* Newton search from guess.  */
 	status = goal_seek_newton (&gnumeric_rate_f, &gnumeric_rate_df,
@@ -1676,8 +1669,8 @@ static GoalSeekStatus
 irr_npv (gnum_float rate, gnum_float *y, void *user_data)
 {
 	gnumeric_irr_t *p = user_data;
-	gnum_float        *values, sum;
-        int            i, n;
+	gnum_float *values, sum;
+        int i, n;
 
 	values = p->values;
 	n = p->n;
@@ -1700,8 +1693,8 @@ static GoalSeekStatus
 irr_npv_df (gnum_float rate, gnum_float *y, void *user_data)
 {
 	gnumeric_irr_t *p = user_data;
-	gnum_float        *values, sum;
-        int            i, n;
+	gnum_float *values, sum;
+        int i, n;
 
 	values = p->values;
 	n = p->n;
@@ -1721,7 +1714,7 @@ gnumeric_irr (FunctionEvalInfo *ei, Value **argv)
 	GoalSeekStatus  status;
 	Value           *result = NULL;
 	gnumeric_irr_t  p;
-	gnum_float         rate0;
+	gnum_float      rate0;
 
 	rate0 = argv[1] ? value_get_as_float (argv[1]) : 0.1;
 
@@ -1736,16 +1729,10 @@ gnumeric_irr (FunctionEvalInfo *ei, Value **argv)
 
 	goal_seek_initialise (&data);
 
-	/* Respect the sign of the guess.  */
-	if (rate0 >= 0) {
-		data.xmin = 0;
-		data.xmax = MIN (data.xmax,
-				 pow (DBL_MAX / 1e10, 1.0 / (p.n + 1)) - 1);
-	} else {
-		data.xmin = MAX (data.xmin,
-				 -pow (DBL_MAX / 1e10, 1.0 / (p.n + 1)) + 1);
-		data.xmax = 0;
-	}
+	data.xmin = MAX (data.xmin,
+			 -pow (DBL_MAX / 1e10, 1.0 / p.n) + 1);
+	data.xmax = MIN (data.xmax,
+			 pow (DBL_MAX / 1e10, 1.0 / p.n) - 1);
 
 	status = goal_seek_newton (&irr_npv, &irr_npv_df, &data, &p, rate0);
 	if (status != GOAL_SEEK_OK) {
@@ -1772,7 +1759,7 @@ gnumeric_irr (FunctionEvalInfo *ei, Value **argv)
 
 static char *help_pv = {
 	N_("@FUNCTION=PV\n"
-	   "@SYNTAX=PV(rate,nper,pmt,fv,type)\n"
+	   "@SYNTAX=PV(rate,nper,pmt[,fv,type])\n"
 	   "@DESCRIPTION="
 	   "PV calculates the present value of an investment. "
 	   "@rate is the periodic interest rate, @nper is the "
@@ -1780,7 +1767,7 @@ static char *help_pv = {
 	   "@pmt is the payment made each period, "
 	   "@fv is the future value and @type is when the payment is made. "
 	   "If @type = 1 then the payment is made at the begining of the "
-	   "period. If @type = 0 it is made at the end of each period."
+	   "period. If @type = 0 (or omitted) it is made at the end of each period."
 	   "@EXAMPLES=\n"
 	   "\n"
 	   "@SEEALSO=FV")
@@ -1796,21 +1783,22 @@ gnumeric_pv (FunctionEvalInfo *ei, Value **argv)
 	rate = value_get_as_float (argv[0]);
 	nper = value_get_as_float (argv[1]);
 	pmt  = value_get_as_float (argv[2]);
-	fv   = value_get_as_float (argv[3]);
-	type = !!value_get_as_int (argv[4]);
+	fv   = argv[3] ? value_get_as_float (argv[3]) : 0;
+	type = argv[4] ? !!value_get_as_int (argv[4]) : 0;
 
-	if (rate <= 0.0)
-		return value_new_error (ei->pos, _("pv - domain error"));
+	/* Special case for which we cannot calculate pvif and/or fvifa.  */
+	if (rate == 0)
+		return value_new_float (-nper * pmt);
 
 	/* Calculate the PVIF and FVIFA */
 	pvif = calculate_pvif (rate, nper);
 	fvifa = calculate_fvifa (rate, nper);
 
-	if ( pvif == 0 )
-	        return value_new_error (ei->pos, gnumeric_err_NUM);
+	if (pvif == 0)
+	        return value_new_error (ei->pos, gnumeric_err_DIV0);
 
-        return value_new_float ( ( -fv - pmt *
-				   ( 1.0 + rate * type ) * fvifa ) / pvif );
+        return value_new_float ((-fv - pmt * (1.0 + rate * type) * fvifa) /
+				pvif);
 }
 
 /***************************************************************************/
@@ -1891,8 +1879,8 @@ gnumeric_xnpv (FunctionEvalInfo *ei, Value **argv)
 {
 	gnum_float rate, *payments = NULL, *dates = NULL;
 	gnum_float sum;
-	int     p_n, d_n, i;
-	Value   *result = NULL;
+	int  p_n, d_n, i;
+	Value *result = NULL;
 
 	rate = value_get_as_float (argv[0]);
 	sum = 0;
@@ -1968,8 +1956,8 @@ static GoalSeekStatus
 xirr_npv (gnum_float rate, gnum_float *y, void *user_data)
 {
 	gnumeric_xirr_t *p = user_data;
-	gnum_float         *values, *dates, sum;
-        int             i, n;
+	gnum_float *values, *dates, sum;
+        int i, n;
 
 	values = p->values;
 	dates = p->dates;
@@ -1995,7 +1983,7 @@ gnumeric_xirr (FunctionEvalInfo *ei, Value **argv)
 	GoalSeekStatus  status;
 	Value           *result = NULL;
 	gnumeric_xirr_t p;
-	gnum_float         rate0;
+	gnum_float      rate0;
 	int             n, d_n;
 
 	goal_seek_initialise (&data);
@@ -2059,8 +2047,8 @@ gnumeric_fv (FunctionEvalInfo *ei, Value **argv)
 	rate = value_get_as_float (argv[0]);
 	nper = value_get_as_float (argv[1]);
 	pmt  = value_get_as_float (argv[2]);
-	pv   = value_get_as_float (argv[3]);
-	type = !!value_get_as_int (argv[4]);
+	pv   = (NULL != argv[3]) ? value_get_as_float (argv[3]) : 0.;
+	type = (NULL != argv[4] && 0 != value_get_as_int (argv[4])) ? 1 : 0;
 
 	pvif = calculate_pvif (rate, nper);
 	fvifa = calculate_fvifa (rate, nper);
@@ -2304,8 +2292,8 @@ static Value *
 gnumeric_fvschedule (FunctionEvalInfo *ei, Value **argv)
 {
 	gnum_float pv, *schedule = NULL;
-	Value   *result = NULL;
-	int     i, n;
+	Value *result = NULL;
+	int i, n;
 
 	pv = value_get_as_float (argv[0]);
 	schedule = collect_floats_value (argv[1], ei->pos,
@@ -2356,7 +2344,7 @@ static char *help_euro = {
 static Value *
 gnumeric_euro (FunctionEvalInfo *ei, Value **argv)
 {
-        char *str = argv[0]->v_str.val->str;
+        const char *str = value_peek_string (argv[0]);
 
 	switch (*str) {
 	case 'A':
@@ -2611,32 +2599,24 @@ static char *help_coupdaybs = {
 static Value *
 gnumeric_coupdaybs (FunctionEvalInfo *ei, Value **argv)
 {
-        GDate   *settlement;
-        GDate   *maturity;
-        int     freq, basis;
-        int     days;
+        GDate *settlement;
+        GDate *maturity;
+        int freq, basis;
+        int days;
 
-	if (argv[0]->type == VALUE_ERROR)
-		return value_duplicate (argv[0]);
-	if (argv[1]->type == VALUE_ERROR)
-		return value_duplicate (argv[1]);
-
-        settlement = datetime_value_to_g(argv[0]);
-        maturity = datetime_value_to_g(argv[1]);
+        settlement = datetime_value_to_g (argv[0]);
+        maturity = datetime_value_to_g (argv[1]);
         freq = value_get_as_int (argv[2]);
-	if (argv[3] != NULL)
-	        basis = value_get_as_int (argv[3]);
-	else
-	        basis = 0;
+	basis = argv[3] ? value_get_as_int (argv[3]) : 0;
 
         if (settlement == NULL || maturity == NULL)
                 return value_new_error (ei->pos, gnumeric_err_VALUE);
 
         if (basis < 0 || basis > 4 || (freq != 1 && freq != 2 && freq != 4)
-	    || g_date_compare(settlement, maturity) > 0)
+	    || g_date_compare (settlement, maturity) > 0)
                 return value_new_error (ei->pos, gnumeric_err_NUM);
 
-        days = coupdaybs(settlement, maturity, freq, basis);
+        days = coupdaybs (settlement, maturity, freq, basis);
 
         if (days < 0)
 	        return value_new_error (ei->pos, "#UNIMPLEMENTED!");
@@ -2661,32 +2641,24 @@ static char *help_coupdays = {
 static Value *
 gnumeric_coupdays (FunctionEvalInfo *ei, Value **argv)
 {
-        GDate   *settlement;
-        GDate   *maturity;
-        int     freq, basis;
+        GDate *settlement;
+        GDate *maturity;
+        int freq, basis;
         gnum_float days;
 
-	if (argv[0]->type == VALUE_ERROR)
-		return value_duplicate (argv[0]);
-	if (argv[1]->type == VALUE_ERROR)
-		return value_duplicate (argv[1]);
-
-        settlement = datetime_value_to_g(argv[0]);
-        maturity = datetime_value_to_g(argv[1]);
+        settlement = datetime_value_to_g (argv[0]);
+        maturity = datetime_value_to_g (argv[1]);
         freq = value_get_as_int (argv[2]);
-	if (argv[3] != NULL)
-	        basis = value_get_as_int (argv[3]);
-	else
-	        basis = 0;
+	basis = argv[3] ? value_get_as_int (argv[3]) : 0;
 
         if (settlement == NULL || maturity == NULL)
                 return value_new_error (ei->pos, gnumeric_err_VALUE);
 
         if (basis < 0 || basis > 4 || (freq != 1 && freq != 2 && freq != 4)
-	    || g_date_compare(settlement, maturity) > 0)
+	    || g_date_compare (settlement, maturity) > 0)
                 return value_new_error (ei->pos, gnumeric_err_NUM);
 
-        days = coupdays(settlement, maturity, freq, basis);
+        days = coupdays (settlement, maturity, freq, basis);
 
         if (days < 0)
 	        return value_new_error (ei->pos, "#UNIMPLEMENTED!");
@@ -2736,24 +2708,16 @@ gnumeric_coupncd (FunctionEvalInfo *ei, Value **argv)
         int     freq, basis;
 	Value   *v;
 
-	if (argv[0]->type == VALUE_ERROR)
-		return value_duplicate (argv[0]);
-	if (argv[1]->type == VALUE_ERROR)
-		return value_duplicate (argv[1]);
-
-        settlement = datetime_value_to_g(argv[0]);
-        maturity = datetime_value_to_g(argv[1]);
+        settlement = datetime_value_to_g (argv[0]);
+        maturity = datetime_value_to_g (argv[1]);
         freq = value_get_as_int (argv[2]);
-	if (argv[3] != NULL)
-	        basis = value_get_as_int (argv[3]);
-	else
-	        basis = 0;
+	basis = argv[3] ? value_get_as_int (argv[3]) : 0;
 
         if (settlement == NULL || maturity == NULL)
                 return value_new_error (ei->pos, gnumeric_err_VALUE);
 
         if (basis < 0 || basis > 4 || (freq != 1 && freq != 2 && freq != 4)
-	    || g_date_compare(settlement, maturity) > 0)
+	    || g_date_compare (settlement, maturity) > 0)
                 return value_new_error (ei->pos, gnumeric_err_NUM);
 
         date = coupncd (settlement, maturity, freq, basis);
@@ -2798,29 +2762,21 @@ static char *help_coupnum = {
 static Value *
 gnumeric_coupnum (FunctionEvalInfo *ei, Value **argv)
 {
-        GDate   *settlement;
-        GDate   *maturity;
-        int     freq, basis;
+        GDate *settlement;
+        GDate *maturity;
+        int freq, basis;
         gnum_float n;
 
-	if (argv[0]->type == VALUE_ERROR)
-		return value_duplicate (argv[0]);
-	if (argv[1]->type == VALUE_ERROR)
-		return value_duplicate (argv[1]);
-
-        settlement = datetime_value_to_g(argv[0]);
-        maturity = datetime_value_to_g(argv[1]);
+        settlement = datetime_value_to_g (argv[0]);
+        maturity = datetime_value_to_g (argv[1]);
         freq = value_get_as_int (argv[2]);
-	if (argv[3] != NULL)
-	        basis = value_get_as_int (argv[3]);
-	else
-	        basis = 0;
+	basis = argv[3] ? value_get_as_int (argv[3]) : 0;
 
         if (settlement == NULL || maturity == NULL)
                 return value_new_error (ei->pos, gnumeric_err_VALUE);
 
         if (basis < 0 || basis > 4 || (freq != 1 && freq != 2 && freq != 4)
-	    || g_date_compare(settlement, maturity) > 0)
+	    || g_date_compare (settlement, maturity) > 0)
                 return value_new_error (ei->pos, gnumeric_err_NUM);
 
         n = coupnum (settlement, maturity, freq, basis);
@@ -3006,7 +2962,7 @@ finance_functions_init (void)
 	def = function_add_args	 (cat, "euro", "s", "currency",
 				  &help_euro,	  gnumeric_euro);
 
-	def = function_add_args	 (cat, "fv", "fffff", "rate,nper,pmt,pv,type",
+	def = function_add_args	 (cat, "fv", "fff|ff", "rate,nper,pmt,pv,type",
 				  &help_fv,	  gnumeric_fv);
 	auto_format_function_result (def, AF_MONETARY);
 
@@ -3090,7 +3046,7 @@ finance_functions_init (void)
 				  "settlement,maturity,issue,rate,yield[,basis]",
 				  &help_pricemat,  gnumeric_pricemat);
 
-	def = function_add_args	 (cat, "pv", "fffff", "rate,nper,pmt,fv,type",
+	def = function_add_args	 (cat, "pv", "fff|ff", "rate,nper,pmt[,fv,type]",
 				  &help_pv,	  gnumeric_pv);
 	auto_format_function_result (def, AF_MONETARY);
 
