@@ -181,10 +181,10 @@ sheet_selection_redraw (Sheet const *sheet)
 		Range const *r = sel->data;
 
 		SHEET_FOREACH_CONTROL (sheet, control,
-			scg_redraw_cell_region (control,
+			sc_redraw_cell_region (control,
 				r->start.col, r->start.row,
 				r->end.col, r->end.row);
-			scg_redraw_headers (control, TRUE, TRUE, r););
+			sc_redraw_headers (control, TRUE, TRUE, r););
 	}
 }
 
@@ -237,14 +237,15 @@ sheet_selection_set_internal (Sheet *sheet,
 	g_return_if_fail (IS_SHEET (sheet));
 	g_return_if_fail (sheet->selections != NULL);
 
-	ss = (Range *)sheet->selections->data;
-
 	new_sel.start.col = MIN(base_col, move_col);
 	new_sel.start.row = MIN(base_row, move_row);
 	new_sel.end.col = MAX(base_col, move_col);
 	new_sel.end.row = MAX(base_row, move_row);
 
+	g_return_if_fail (range_is_sane (&new_sel));
+
 	sheet_merge_find_container (sheet, &new_sel);
+	ss = (Range *)sheet->selections->data;
 	if (!just_add_it && range_equal (ss, &new_sel))
 		return;
 
@@ -489,14 +490,10 @@ sheet_selection_reset (Sheet *sheet)
 static void
 reference_append (GString *result_str, CellPos const *pos)
 {
-	char *row_string = g_strdup_printf ("%d", pos->row);
-
 	g_string_append_c (result_str, '$');
 	g_string_append (result_str, col_name (pos->col));
 	g_string_append_c (result_str, '$');
-	g_string_append (result_str, row_string);
-
-	g_free (row_string);
+	g_string_append (result_str, row_name (pos->row));
 }
 
 char *
@@ -658,12 +655,12 @@ selection_get_ranges (Sheet const *sheet, gboolean allow_intersection)
 				segments_intersect (a->start.row, a->end.row,
 						    b->start.row, b->end.row);
 #ifdef DEBUG_SELECTION
-			fprintf (stderr, "row = %d\na = %d", row_intersect, a->start.row +1);
+			fprintf (stderr, "row = %d\na = %s", row_intersect, row_name (a->start.row));
 			if (a->start.row != a->end.row)
-				fprintf (stderr, " -> %d", a->end.row +1);
-			fprintf (stderr, "\nb = %d", b->start.row +1);
+				fprintf (stderr, " -> %s", row_name (a->end.row));
+			fprintf (stderr, "\nb = %s", row_name (b->start.row));
 			if (b->start.row != b->end.row)
-				fprintf (stderr, " -> %d\n", b->end.row +1);
+				fprintf (stderr, " -> %s\n", row_name (b->end.row));
 			else
 				fputc ('\n', stderr);
 #endif
@@ -924,11 +921,11 @@ cb_range_to_string (Sheet *sheet, Range const *r, void *closure)
 	if (res->include_sheet_name_prefix)
 		g_string_sprintfa (res->str, "%s!", sheet->name_quoted);
 
-	g_string_sprintfa (res->str, "$%s$%d",
-			   col_name (r->start.col), r->start.row+1);
+	g_string_sprintfa (res->str, "$%s$%s",
+			   col_name (r->start.col), row_name (r->start.row));
 	if ((r->start.col != r->end.col) || (r->start.row != r->end.row))
-		g_string_sprintfa (res->str, ":$%s$%d",
-				   col_name (r->end.col), r->end.row+1);
+		g_string_sprintfa (res->str, ":$%s$%s",
+				   col_name (r->end.col), row_name (r->end.row));
 }
 
 char *
