@@ -1,4 +1,4 @@
-/* vim: set sw=8: */
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * sheet-control-gui.c: Implements a graphic control for a sheet.
  *
@@ -8,7 +8,7 @@
  */
 #include <config.h>
 
-#include "sheet-control-gui.h"
+#include "sheet-control-gui-priv.h"
 #include "item-bar.h"
 #define GNUMERIC_ITEM "SCG"
 #include "item-debug.h"
@@ -50,15 +50,20 @@
 #include <string.h>
 #include <gal/widgets/e-cursors.h>
 
-static GtkTableClass *scg_parent_class;
+static SheetControlClass *scg_parent_class;
 
-void
-scg_redraw_all (SheetControlGUI *scg)
+static void scg_ant                    (SheetControl *sc);
+static void scg_unant                  (SheetControl *sc);
+
+static void
+scg_redraw_all (SheetControl *sc)
 {
+	SheetControlGUI *scg = (SheetControlGUI *)sc;
+
 	g_return_if_fail (IS_SHEET_CONTROL_GUI (scg));
 
 	gnome_canvas_request_redraw (
-		GNOME_CANVAS (scg->canvas),
+		GNOME_CANVAS (scg->gsheet),
 		0, 0, INT_MAX, INT_MAX);
 	gnome_canvas_request_redraw (
 		GNOME_CANVAS (scg->col_canvas),
@@ -71,18 +76,19 @@ scg_redraw_all (SheetControlGUI *scg)
 /*
  * Redraw selected range, do not honour spans
  */
-void
-scg_redraw_cell_region (SheetControlGUI *scg,
+static void
+scg_redraw_cell_region (SheetControl *sc,
 			int start_col, int start_row,
 			int end_col, int end_row)
 {
+	SheetControlGUI *scg = (SheetControlGUI *)sc;
 	GnumericSheet *gsheet;
 	GnomeCanvas *canvas;
 	int x1, y1, x2, y2;
 
 	g_return_if_fail (IS_SHEET_CONTROL_GUI (scg));
 
-	gsheet = GNUMERIC_SHEET (scg->canvas);
+	gsheet = GNUMERIC_SHEET (scg->gsheet);
 	g_return_if_fail (GNUMERIC_IS_SHEET (gsheet));
 	canvas = GNOME_CANVAS (gsheet);
 
@@ -115,23 +121,24 @@ scg_redraw_cell_region (SheetControlGUI *scg,
 		: INT_MAX;
 
 #if 0
-	fprintf (stderr, "%s%d:", col_name(min_col), first_row+1);
-	fprintf (stderr, "%s%d\n", col_name(max_col), last_row+1);
+	fprintf (stderr, "%s%s:", col_name (min_col), row_name (first_row));
+	fprintf (stderr, "%s%s\n", col_name (max_col), row_name (last_row));
 #endif
 
 	gnome_canvas_request_redraw (GNOME_CANVAS (gsheet), x1-2, y1-2, x2, y2);
 }
 
-void
-scg_redraw_headers (SheetControlGUI *scg,
+static void
+scg_redraw_headers (SheetControl *sc,
 		    gboolean const col, gboolean const row,
 		    Range const * r /* optional == NULL */)
 {
+	SheetControlGUI *scg = (SheetControlGUI *)sc;
 	GnumericSheet *gsheet;
 
 	g_return_if_fail (IS_SHEET_CONTROL_GUI (scg));
 
-	gsheet = GNUMERIC_SHEET (scg->canvas);
+	gsheet = GNUMERIC_SHEET (scg->gsheet);
 
 	if (col) {
 		int left = 0, right = INT_MAX-1;
@@ -180,15 +187,16 @@ scg_redraw_headers (SheetControlGUI *scg,
 	}
 }
 
-void
-scg_update_cursor_pos (SheetControlGUI *scg)
+static void
+scg_update_cursor_pos (SheetControl *sc)
 {
+	SheetControlGUI *scg = (SheetControlGUI *)sc;
 	GList *l;
 	GnumericSheet *gsheet;
 
 	g_return_if_fail (IS_SHEET_CONTROL_GUI (scg));
 
-	gsheet = GNUMERIC_SHEET (scg->canvas);
+	gsheet = GNUMERIC_SHEET (scg->gsheet);
 
 	/* Repsition the selection cursor */
 	item_cursor_reposition (gsheet->item_cursor);
@@ -202,9 +210,11 @@ scg_update_cursor_pos (SheetControlGUI *scg)
  * scg_resize :
  * @scg :
  */
-void
-scg_resize (SheetControlGUI *scg)
+static void
+scg_resize (SheetControl *sc)
 {
+	SheetControlGUI *scg = (SheetControlGUI *)sc;
+	Sheet *sheet = sc->sheet;
 	GnumericSheet *gsheet;
 	ItemBar *col_item, *row_item;
 	int h, w;
@@ -212,8 +222,8 @@ scg_resize (SheetControlGUI *scg)
 
 	g_return_if_fail (IS_SHEET_CONTROL_GUI (scg));
 
-	gsheet = GNUMERIC_SHEET (scg->canvas);
-	zoom = scg->sheet->last_zoom_factor_used;
+	gsheet = GNUMERIC_SHEET (scg->gsheet);
+	zoom = sheet->last_zoom_factor_used;
 
 	g_return_if_fail (IS_SHEET_CONTROL_GUI (scg));
 
@@ -241,30 +251,31 @@ scg_resize (SheetControlGUI *scg)
 		/* Ensure that the current cell remains visible when we zoom */
 		gnumeric_sheet_make_cell_visible
 			(gsheet,
-			 scg->sheet->edit_pos.col,
-			 scg->sheet->edit_pos.row,
+			 sheet->edit_pos.col,
+			 sheet->edit_pos.row,
 			 TRUE);
 
-	scg_update_cursor_pos (scg);
+	scg_update_cursor_pos (sc);
 }
 
 void
-scg_set_zoom_factor (SheetControlGUI *scg)
+scg_set_zoom_factor (SheetControl *sc)
 {
+	SheetControlGUI *scg = (SheetControlGUI *)sc;
 	GnumericSheet *gsheet;
 	double zoom;
 
 	g_return_if_fail (IS_SHEET_CONTROL_GUI (scg));
 
-	gsheet = GNUMERIC_SHEET (scg->canvas);
-	zoom = scg->sheet->last_zoom_factor_used;
+	gsheet = GNUMERIC_SHEET (scg->gsheet);
+	zoom = sc->sheet->last_zoom_factor_used;
 
 	/* Set pixels_per_unit before the font.  The item bars look here for the number */
 	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (gsheet), zoom);
 	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (scg->col_canvas), zoom);
 	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (scg->row_canvas), zoom);
 
-	scg_resize (scg);
+	scg_resize (sc);
 }
 
 static void
@@ -299,12 +310,13 @@ new_canvas_bar (SheetControlGUI *scg, gboolean is_col_header, GnomeCanvasItem **
 
 /* Manages the scrollbar dimensions and paging parameters. */
 void
-scg_scrollbar_config (SheetControlGUI const *scg)
+scg_scrollbar_config (SheetControl const *sc)
 {
+	SheetControlGUI *scg = SHEET_CONTROL_GUI (sc);
 	GtkAdjustment *va = GTK_ADJUSTMENT (scg->va);
 	GtkAdjustment *ha = GTK_ADJUSTMENT (scg->ha);
-	GnumericSheet *gsheet = GNUMERIC_SHEET (scg->canvas);
-	Sheet         *sheet = scg->sheet;
+	GnumericSheet *gsheet = GNUMERIC_SHEET (scg->gsheet);
+	Sheet         *sheet = sc->sheet;
 	int const last_col = gsheet->col.last_full;
 	int const last_row = gsheet->row.last_full;
 	int max_col = last_col;
@@ -375,15 +387,16 @@ scg_size_allocate (GtkWidget *widget, GtkAllocation *alloc,
 	 */
 	scg_make_edit_pos_visible (scg);
 #endif
-	scg_scrollbar_config (scg);
+	scg_scrollbar_config ((SheetControl *) scg);
 }
 
 void
 scg_colrow_size_set (SheetControlGUI *scg,
 		     gboolean is_cols, int index, int new_size_pixels)
 {
-	WorkbookControl *wbc = WORKBOOK_CONTROL (scg->wbcg);
-	Sheet *sheet = scg->sheet;
+	SheetControl *sc = (SheetControl *) scg;
+	WorkbookControl *wbc = sc->wbc;
+	Sheet *sheet = sc->sheet;
 
 	/* If all cols/rows in the selection are completely selected
 	 * then resize all of them, otherwise just resize the selected col/row.
@@ -399,13 +412,14 @@ scg_colrow_size_set (SheetControlGUI *scg,
 static void
 scg_select_all (SheetControlGUI *scg)
 {
-	Sheet *sheet = scg->sheet;
+	SheetControl *sc = (SheetControl *) scg;
+	Sheet *sheet = sc->sheet;
 	gboolean const rangesel = wbcg_rangesel_possible (scg->wbcg);
 
 	if (!rangesel) {
 		if (!wbcg_edit_has_guru (scg->wbcg)) {
 			wbcg_edit_finish (scg->wbcg, FALSE);
-			cmd_select_all (scg->sheet);
+			cmd_select_all (sheet);
 		}
 	} else {
 		if (!scg->rangesel.active)
@@ -420,12 +434,13 @@ void
 scg_colrow_select (SheetControlGUI *scg, gboolean is_cols,
 		   int index, int modifiers)
 {
-	Sheet *sheet = scg->sheet;
-	GnumericSheet *gsheet = GNUMERIC_SHEET (scg->canvas);
+	SheetControl *sc = (SheetControl *) scg;
+	Sheet *sheet = sc->sheet;
+	GnumericSheet *gsheet = GNUMERIC_SHEET (scg->gsheet);
 	gboolean const rangesel = wbcg_rangesel_possible (scg->wbcg);
 
 	if (!rangesel)
-		wbcg_edit_finish (scg->wbcg, FALSE);
+		wbcg_edit_finish (scg->wbcg, TRUE);
 
 	if (rangesel && !scg->rangesel.active)
 		scg_rangesel_start (scg, index, index);
@@ -483,10 +498,10 @@ static void
 vertical_scroll_offset_changed (GtkAdjustment *adj, int top, int is_hint,
 				SheetControlGUI *scg)
 {
-	GnumericSheet  *gsheet = GNUMERIC_SHEET (scg->canvas);
+	GnumericSheet  *gsheet = GNUMERIC_SHEET (scg->gsheet);
 
 	if (is_hint) {
-		char *buffer = g_strdup_printf (_("Row: %d"), top + 1);
+		char *buffer = g_strdup_printf (_("Row: %s"), row_name (top));
 		wb_control_gui_set_status_text (scg->wbcg, buffer);
 		g_free (buffer);
 	} else {
@@ -499,7 +514,7 @@ static void
 horizontal_scroll_offset_changed (GtkAdjustment *adj, int left, int is_hint,
 				  SheetControlGUI *scg)
 {
-	GnumericSheet  *gsheet = GNUMERIC_SHEET (scg->canvas);
+	GnumericSheet  *gsheet = GNUMERIC_SHEET (scg->gsheet);
 
 	if (is_hint) {
 		char *buffer = g_strdup_printf (_("Column: %s"), col_name (left));
@@ -511,17 +526,31 @@ horizontal_scroll_offset_changed (GtkAdjustment *adj, int left, int is_hint,
 	}
 }
 
+static void
+cb_table_destroy (GtkObject *table, SheetControlGUI *scg)
+{
+	SheetControl *sc = (SheetControl *) scg;
+
+	scg_mode_edit (sc);	/* finish any object edits */
+	scg_unant (sc);		/* Make sure that everything is unanted */
+
+ 	if (scg->wbcg) {
+		GtkWindow *toplevel = wb_control_gui_toplevel (scg->wbcg);
+
+		if (toplevel && (toplevel->focus_widget == scg->gsheet))
+			gtk_window_set_focus (toplevel, NULL);
+	}
+	scg->table = NULL;
+	scg->gsheet = NULL;
+}
+
 static void
 scg_init (SheetControlGUI *scg)
 {
-	GtkTable *table = GTK_TABLE (scg);
-
-	scg->sheet = NULL;
+	((SheetControl *) scg)->sheet = NULL;
 	scg->slide_handler = NULL;
 	scg->slide_data = NULL;
 	scg->sliding = -1;
-	table->homogeneous = FALSE;
-	gtk_table_resize (table, 4, 4);
 
 	scg->comment.selected = NULL;
 	scg->comment.item = NULL;
@@ -537,39 +566,41 @@ static void
 scg_construct (SheetControlGUI *scg)
 {
 	GnomeCanvasGroup *root_group;
-	GtkTable  *outer_table = GTK_TABLE (scg);
-	GtkTable  *table = GTK_TABLE (gtk_table_new (2, 2, FALSE));
-	Sheet *sheet = scg->sheet;
+	GtkTable  *inner_table;
+	SheetControl *sc = (SheetControl *) scg;
+	Sheet *sheet = sc->sheet;
 	int i;
 
+	scg->table = GTK_TABLE (gtk_table_new (4, 4, FALSE));
+	inner_table = GTK_TABLE (gtk_table_new (2, 2, FALSE));
 	scg->col_canvas = new_canvas_bar (scg, TRUE, &scg->col_item);
-	gtk_table_attach (table, GTK_WIDGET (scg->col_canvas),
+	gtk_table_attach (inner_table, GTK_WIDGET (scg->col_canvas),
 			  1, 2, 0, 1,
 			  GTK_EXPAND | GTK_FILL | GTK_SHRINK,
 			  GTK_FILL,
 			  0, 0);
 
 	scg->row_canvas = new_canvas_bar (scg, FALSE, &scg->row_item);
-	gtk_table_attach (table, GTK_WIDGET (scg->row_canvas),
+	gtk_table_attach (inner_table, GTK_WIDGET (scg->row_canvas),
 			  0, 1, 1, 2,
 			  GTK_FILL, GTK_EXPAND | GTK_FILL | GTK_SHRINK,
 			  0, 0);
 
-	scg->canvas = gnumeric_sheet_new (scg);
+	scg->gsheet = gnumeric_sheet_new (scg);
 	gtk_signal_connect_after (
-		GTK_OBJECT (scg), "size_allocate",
+		GTK_OBJECT (scg->table), "size_allocate",
 		GTK_SIGNAL_FUNC (scg_size_allocate), scg);
-	gtk_table_attach (table, scg->canvas,
+	gtk_table_attach (inner_table, scg->gsheet,
 			  1, 2, 1, 2,
 			  GTK_EXPAND | GTK_FILL | GTK_SHRINK,
 			  GTK_EXPAND | GTK_FILL | GTK_SHRINK,
 			  0, 0);
-	gtk_widget_show (scg->canvas);
+	gtk_widget_show (scg->gsheet);
 
 	/* The select-all button */
 	scg->select_all_btn = gtk_button_new ();
 	GTK_WIDGET_UNSET_FLAGS (scg->select_all_btn, GTK_CAN_FOCUS);
-	gtk_table_attach (table, scg->select_all_btn, 0, 1, 0, 1,
+	gtk_table_attach (inner_table, scg->select_all_btn, 0, 1, 0, 1,
 			  GTK_FILL, GTK_FILL, 0, 0);
 	gtk_signal_connect (GTK_OBJECT (scg->select_all_btn), "clicked",
 			    GTK_SIGNAL_FUNC (button_select_all), scg);
@@ -586,21 +617,21 @@ scg_construct (SheetControlGUI *scg)
 			    GTK_SIGNAL_FUNC (horizontal_scroll_offset_changed),
 			    scg);
 
-	gtk_table_attach (outer_table, GTK_WIDGET (table),	0, 1, 0, 1,
+	gtk_table_attach (scg->table, GTK_WIDGET (inner_table), 0, 1, 0, 1,
 			  GTK_EXPAND | GTK_FILL | GTK_SHRINK,
 			  GTK_EXPAND | GTK_FILL | GTK_SHRINK,
 			  0, 0);
-	gtk_table_attach (outer_table, scg->vs,			1, 2, 0, 1,
+	gtk_table_attach (scg->table, scg->vs,			1, 2, 0, 1,
 			  GTK_FILL,
 			  GTK_EXPAND | GTK_FILL | GTK_SHRINK,
 			  0, 0);
-	gtk_table_attach (outer_table, scg->hs,			0, 1, 1, 2,
+	gtk_table_attach (scg->table, scg->hs,			0, 1, 1, 2,
 			  GTK_EXPAND | GTK_FILL | GTK_SHRINK,
 			  GTK_FILL,
 			  0, 0);
 
 	/* cursor group */
-	root_group = GNOME_CANVAS_GROUP (GNOME_CANVAS (scg->canvas)->root);
+	root_group = GNOME_CANVAS_GROUP (GNOME_CANVAS (scg->gsheet)->root);
 	scg->anted_group = GNOME_CANVAS_GROUP (
 		gnome_canvas_item_new (
 			root_group,
@@ -622,7 +653,12 @@ scg_construct (SheetControlGUI *scg)
 	while (i-- > 0)
 		scg->control_points[i] = NULL;
 
-	scg_ant (scg);
+	scg_ant (sc);
+
+	/* Store some useful information */
+	gtk_object_set_data (GTK_OBJECT (scg->table), SHEET_CONTROL_KEY, scg);
+	gtk_signal_connect (GTK_OBJECT (scg->table), "destroy",
+			    GTK_SIGNAL_FUNC (cb_table_destroy), scg);
 }
 
 /* FIXME : Should be SheetControl */
@@ -634,7 +670,7 @@ sheet_control_gui_new (Sheet *sheet)
 	g_return_val_if_fail (IS_SHEET (sheet), NULL);
 
 	scg = gtk_type_new (sheet_control_gui_get_type ());
-	scg->sheet = sheet;
+	((SheetControl *) scg)->sheet = sheet;
 	scg->tip = NULL;
 
 	scg_construct (scg);
@@ -646,48 +682,28 @@ static void
 scg_destroy (GtkObject *object)
 {
 	SheetControlGUI *scg = SHEET_CONTROL_GUI (object);
-
-	scg_mode_edit (scg); /* finish any object edits */
-	scg_unant (scg); /* Make sure that everything is unanted */
+	SheetControl *sc = (SheetControl *) scg;
 
 	/* Add shutdown code here */
 	if (scg->tip)
 		gtk_object_unref (GTK_OBJECT (scg->tip));
 
-	if (scg->sheet)
-		sheet_detach_scg (scg);
+	if (sc->sheet)
+		sheet_detach_control (sc);
 
-	if (scg->wbcg) {
-		GtkWindow *toplevel = wb_control_gui_toplevel (scg->wbcg);
+	if (scg->table)
+		gtk_object_unref (GTK_OBJECT (scg->table));
+	
+	/* FIXME : Should we be pedantic and clear the control points ? */
 
-		if (toplevel && (toplevel->focus_widget == scg->canvas))
-			gtk_window_set_focus (toplevel, NULL);
-	}
-
-	/* FIXME : Should we be pedantic and
-	 * 1) clear the control points
-	 * 2) remove ourselves from the sheets list of views ?
-	 */
 	if (GTK_OBJECT_CLASS (scg_parent_class)->destroy)
 		(*GTK_OBJECT_CLASS (scg_parent_class)->destroy)(object);
 }
 
 static void
-scg_class_init (SheetControlGUIClass *Class)
-{
-	GtkObjectClass *object_class;
-
-	object_class = (GtkObjectClass *) Class;
-	scg_parent_class = gtk_type_class (gtk_table_get_type ());
-	object_class->destroy = scg_destroy;
-}
-
-GNUMERIC_MAKE_TYPE (sheet_control_gui, "SheetControlGUI", SheetControlGUI,
-		    scg_class_init, scg_init, gtk_table_get_type ())
-
-void
-scg_unant (SheetControlGUI *scg)
+scg_unant (SheetControl *sc)
 {
+	SheetControlGUI *scg = (SheetControlGUI *)sc;
 	GList *l;
 
 	g_return_if_fail (IS_SHEET_CONTROL_GUI (scg));
@@ -702,17 +718,18 @@ scg_unant (SheetControlGUI *scg)
 	scg->anted_cursors = NULL;
 }
 
-void
-scg_ant (SheetControlGUI *scg)
+static void
+scg_ant (SheetControl *sc)
 {
+	SheetControlGUI *scg = (SheetControlGUI *)sc;
 	GList *l;
 
 	g_return_if_fail (IS_SHEET_CONTROL_GUI (scg));
 
 	if (scg->anted_cursors)
-		scg_unant (scg);
+		scg_unant (sc);
 
-	for (l = scg->sheet->ants; l; l = l->next){
+	for (l = sc->sheet->ants; l; l = l->next){
 		Range *ss = l->data;
 		ItemCursor *item_cursor;
 
@@ -728,9 +745,10 @@ scg_ant (SheetControlGUI *scg)
 }
 
 void
-scg_adjust_preferences (SheetControlGUI *scg)
+scg_adjust_preferences (SheetControl *sc)
 {
-	Sheet const *sheet = scg->sheet;
+	SheetControlGUI *scg = SHEET_CONTROL_GUI (sc);
+	Sheet const *sheet = sc->sheet;
 
 	if (sheet->hide_col_header)
 		gtk_widget_hide (GTK_WIDGET (scg->col_canvas));
@@ -747,8 +765,8 @@ scg_adjust_preferences (SheetControlGUI *scg)
 	else
 		gtk_widget_show (scg->select_all_btn);
 
-	if (scg->wbcg != NULL) {
-		WorkbookView *wbv = wb_control_view (WORKBOOK_CONTROL (scg->wbcg));
+	if (sc->wbc != NULL) {
+		WorkbookView *wbv = wb_control_view (sc->wbc);
 		if (wbv->show_horizontal_scrollbar)
 			gtk_widget_show (scg->hs);
 		else
@@ -793,7 +811,7 @@ static gint
 scg_sliding_callback (gpointer data)
 {
 	SheetControlGUI *scg = data;
-	GnumericSheet *gsheet = GNUMERIC_SHEET (scg->canvas);
+	GnumericSheet *gsheet = GNUMERIC_SHEET (scg->gsheet);
 	gboolean change = FALSE;
 	int col, row;
 
@@ -885,7 +903,7 @@ scg_start_sliding (SheetControlGUI *scg,
 		   gpointer user_data,
 		   int col, int row, int dx, int dy)
 {
-	GnumericSheet *gsheet = GNUMERIC_SHEET (scg->canvas);
+	GnumericSheet *gsheet = GNUMERIC_SHEET (scg->gsheet);
 
 	/* Do not slide off the edge */
 	if (((dx == 0) ||
@@ -933,13 +951,14 @@ enum {
 	CONTEXT_ROW_UNHIDE
 };
 static gboolean
-context_menu_hander (GnumericPopupMenuElement const *element,
+context_menu_handler (GnumericPopupMenuElement const *element,
 		     gpointer user_data)
 {
 	SheetControlGUI *scg = user_data;
-	Sheet *sheet = scg->sheet;
+	SheetControl *sc = (SheetControl *) scg;
+	Sheet *sheet = sc->sheet;
 	WorkbookControlGUI *wbcg = scg->wbcg;
-	WorkbookControl *wbc = WORKBOOK_CONTROL (wbcg);
+	WorkbookControl *wbc = sc->wbc;
 
 	g_return_val_if_fail (element != NULL, TRUE);
 	g_return_val_if_fail (sheet != NULL, TRUE);
@@ -1000,6 +1019,8 @@ void
 scg_context_menu (SheetControlGUI *scg, GdkEventButton *event,
 		  gboolean is_col, gboolean is_row)
 {
+	SheetControl *sc = (SheetControl *) scg;
+	
 	enum {
 		CONTEXT_DISPLAY_FOR_CELLS = 1,
 		CONTEXT_DISPLAY_FOR_ROWS = 2,
@@ -1096,7 +1117,7 @@ scg_context_menu (SheetControlGUI *scg, GdkEventButton *event,
 	 * whole row or a whole column and disable the insert/delete col/row menu items
 	 * accordingly
 	 */
-	for (l = scg->sheet->selections; l != NULL; l = l->next) {
+	for (l = sc->sheet->selections; l != NULL; l = l->next) {
 		Range const *r = l->data;
 
 		if (r->start.row == 0 && r->end.row == SHEET_MAX_ROWS - 1)
@@ -1106,7 +1127,7 @@ scg_context_menu (SheetControlGUI *scg, GdkEventButton *event,
 			sensitivity_filter |= CONTEXT_DISABLE_FOR_COLS;
 	}
 
-	gnumeric_create_popup_menu (popup_elements, &context_menu_hander,
+	gnumeric_create_popup_menu (popup_elements, &context_menu_handler,
 				    scg, display_filter,
 				    sensitivity_filter, event);
 }
@@ -1114,19 +1135,25 @@ scg_context_menu (SheetControlGUI *scg, GdkEventButton *event,
 static gboolean
 cb_redraw_sel (Sheet *sheet, Range const *r, gpointer user_data)
 {
-	SheetControlGUI *scg = user_data;
-	scg_redraw_cell_region (scg,
+	SheetControl *sc = user_data;
+	scg_redraw_cell_region (sc,
 		r->start.col, r->start.row, r->end.col, r->end.row);
-	scg_redraw_headers (scg, TRUE, TRUE, r);
+	scg_redraw_headers (sc, TRUE, TRUE, r);
 	return TRUE;
 }
 
 static void
 scg_cursor_visible (SheetControlGUI *scg, gboolean is_visible)
 {
-	GnumericSheet *gsheet = GNUMERIC_SHEET (scg->canvas);
+	GnumericSheet *gsheet;
+	SheetControl *sc = (SheetControl *) scg;
+
+	if (!scg->gsheet)
+		return;
+	
+	gsheet = GNUMERIC_SHEET (scg->gsheet);
 	item_cursor_set_visibility (gsheet->item_cursor, is_visible);
-	selection_foreach_range (scg->sheet, TRUE, cb_redraw_sel, scg);
+	selection_foreach_range (sc->sheet, TRUE, cb_redraw_sel, sc);
 }
 
 /***************************************************************************/
@@ -1191,15 +1218,17 @@ scg_mode_clear (SheetControlGUI *scg)
 
 /*
  * scg_mode_edit:
- * @scg:  The sheet
+ * @sc:  The sheet control
  *
  * Put @sheet into the standard state 'edit mode'.  This shuts down
  * any object editing and frees any objects that are created but not
  * realized.
  */
 void
-scg_mode_edit (SheetControlGUI *scg)
+scg_mode_edit (SheetControl *sc)
 {
+	SheetControlGUI *scg = (SheetControlGUI *)sc;
+	
 	g_return_if_fail (IS_SHEET_CONTROL_GUI (scg));
 
 	scg_mode_clear (scg);
@@ -1259,7 +1288,7 @@ display_object_menu (SheetObject *so, GnomeCanvasItem *view, GdkEvent *event)
 
 	scg_mode_edit_object (scg, so);
 	menu = GTK_MENU (gtk_menu_new ());
-	SHEET_OBJECT_CLASS (GTK_OBJECT(so)->klass)->populate_menu (so, view, menu);
+	SHEET_OBJECT_CLASS (GTK_OBJECT(so)->klass)->populate_menu (so, GTK_OBJECT (view), menu);
 
 	gtk_widget_show_all (GTK_WIDGET (menu));
 	gnumeric_popup_menu (menu, &event->button);
@@ -1312,6 +1341,8 @@ scg_object_move (SheetControlGUI *scg, SheetObject *so,
 		g_warning ("Should not happen %d", idx);
 	}
 
+	sheet_object_direction_set (so, new_coords);
+	
 	/* Tell the object to update its co-ordinates */
 	scg_object_update_bbox (scg, so, so_view, new_coords);
 }
@@ -1321,14 +1352,14 @@ cb_slide_handler (SheetControlGUI *scg, int col, int row, gpointer user)
 {
 	int x, y;
 	gdouble new_x, new_y;
-	GnumericSheet   *gsheet  = GNUMERIC_SHEET (scg->canvas);
+	GnumericSheet   *gsheet  = GNUMERIC_SHEET (scg->gsheet);
 	GtkObject *view = sheet_object_get_view (scg->current_object, scg);
 
 	x = scg_colrow_distance_get (scg, TRUE, gsheet->col.first, col);
 	x += gsheet->col_offset.first;
 	y = scg_colrow_distance_get (scg, FALSE, gsheet->row.first, row);
 	y += gsheet->row_offset.first;
-	gnome_canvas_c2w (GNOME_CANVAS (scg->canvas), x, y, &new_x, &new_y);
+	gnome_canvas_c2w (GNOME_CANVAS (scg->gsheet), x, y, &new_x, &new_y);
 	scg_object_move (scg, scg->current_object, GNOME_CANVAS_ITEM (view),
 			 user, new_x, new_y);
 
@@ -1390,8 +1421,8 @@ cb_control_point_event (GnomeCanvasItem *ctrl_pt, GdkEvent *event,
 		break;
 
 	case GDK_MOTION_NOTIFY: {
-		GnomeCanvas *canvas = GNOME_CANVAS (scg->canvas);
-		GnumericSheet *gsheet = GNUMERIC_SHEET (scg->canvas);
+		GnomeCanvas *canvas = GNOME_CANVAS (scg->gsheet);
+		GnumericSheet *gsheet = GNUMERIC_SHEET (scg->gsheet);
 		int col, row, x, y, left, top, width, height;
 
 		if (scg->drag_object != so)
@@ -1401,8 +1432,8 @@ cb_control_point_event (GnomeCanvasItem *ctrl_pt, GdkEvent *event,
 				  &x, &y);
 		gnome_canvas_get_scroll_offsets (canvas, &left, &top);
 
-		width = scg->canvas->allocation.width;
-		height = scg->canvas->allocation.height;
+		width  = scg->gsheet->allocation.width;
+		height = scg->gsheet->allocation.height;
 
 		col = gnumeric_sheet_find_col (gsheet, x, NULL);
 		row = gnumeric_sheet_find_row (gsheet, y, NULL);
@@ -1506,13 +1537,18 @@ set_item_x_y (SheetControlGUI *scg, GtkObject *so_view, int idx,
 		       NULL);
 }
 
+#define normalize_high_low(d1,d2) if (d1<d2){ double tmp=d1; d1=d2; d2=tmp;}
+
 static void
 set_acetate_coords (SheetControlGUI *scg, GtkObject *so_view,
 		    double l, double t, double r, double b)
 {
-	l -= 10.; r += 10.;
-	t -= 10.; b += 10.;
+	normalize_high_low (r,l);
+	normalize_high_low (b,t);
 
+	t -= 10.; b += 10.;
+	l -= 10.; r += 10.;
+	
 	if (scg->control_points [8] == NULL) {
 		GnomeCanvasItem *item;
 		GtkWidget *event_box = gtk_event_box_new ();
@@ -1577,7 +1613,7 @@ scg_object_update_bbox (SheetControlGUI *scg, SheetObject *so,
 		scg_object_calc_position (scg, so, new_coords);
 	else
 		scg_object_view_position (scg, so, scg->object_coords);
-
+	
 	l = scg->object_coords [0];
 	t = scg->object_coords [1];
 	r = scg->object_coords [2];
@@ -1613,13 +1649,14 @@ calc_obj_place (GnumericSheet *gsheet, int pixel, gboolean is_col,
 	int origin;
 	int colrow;
 	ColRowInfo *cri;
+	Sheet *sheet = ((SheetControl *) gsheet->scg)->sheet;
 
 	if (is_col) {
 		colrow = gnumeric_sheet_find_col (gsheet, pixel, &origin);
-		cri = sheet_col_get_info (gsheet->scg->sheet, colrow);
+		cri = sheet_col_get_info (sheet, colrow);
 	} else {
 		colrow = gnumeric_sheet_find_row (gsheet, pixel, &origin);
-		cri = sheet_row_get_info (gsheet->scg->sheet, colrow);
+		cri = sheet_row_get_info (sheet, colrow);
 	}
 
 	/* TODO : handle other anchor types */
@@ -1632,7 +1669,7 @@ calc_obj_place (GnumericSheet *gsheet, int pixel, gboolean is_col,
 void
 scg_object_calc_position (SheetControlGUI *scg, SheetObject *so, double const *coords)
 {
-	GnumericSheet *gsheet = GNUMERIC_SHEET (scg->canvas);
+	GnumericSheet *gsheet = GNUMERIC_SHEET (scg->gsheet);
 	int	i, pixels [4];
 	float	fraction [4];
 	double	tmp [4];
@@ -1656,10 +1693,10 @@ scg_object_calc_position (SheetControlGUI *scg, SheetObject *so, double const *c
 	for (i = 4; i-- > 0 ;)
 		scg->object_coords [i] = coords [i];
 
-	gnome_canvas_w2c (GNOME_CANVAS (scg->canvas),
+	gnome_canvas_w2c (GNOME_CANVAS (scg->gsheet),
 			  tmp [0], tmp [1],
 			  pixels +0, pixels + 1);
-	gnome_canvas_w2c (GNOME_CANVAS (scg->canvas),
+	gnome_canvas_w2c (GNOME_CANVAS (scg->gsheet),
 			  tmp [2], tmp [3],
 			  pixels +2, pixels + 3);
 	range.start.col = calc_obj_place (gsheet, pixels [0], TRUE,
@@ -1677,15 +1714,25 @@ scg_object_calc_position (SheetControlGUI *scg, SheetObject *so, double const *c
 void
 scg_object_view_position (SheetControlGUI *scg, SheetObject *so, double *coords)
 {
+	SheetObjectDirection direction;
 	int pixels [4];
 
 	sheet_object_position_pixels (so, scg, pixels);
-	gnome_canvas_c2w (GNOME_CANVAS (scg->canvas),
-			  pixels [0], pixels [1],
-			  coords +0, coords + 1);
-	gnome_canvas_c2w (GNOME_CANVAS (scg->canvas),
-			  pixels [2], pixels [3],
-			  coords +2, coords + 3);
+
+	direction = so->direction;
+	if (direction == SO_DIR_UNKNOWN)
+		direction = SO_DIR_DOWN_RIGHT;
+
+	gnome_canvas_c2w (
+		GNOME_CANVAS (scg->gsheet),
+		pixels [direction & SO_DIR_LEFT_MASK  ? 2 : 0],
+		pixels [direction & SO_DIR_DOWN_MASK  ? 1 : 3],
+		coords +0, coords + 1);
+	gnome_canvas_c2w (
+		GNOME_CANVAS (scg->gsheet),
+		pixels [direction & SO_DIR_LEFT_MASK  ? 0 : 2],
+		pixels [direction & SO_DIR_DOWN_MASK  ? 3 : 1],
+		coords +2, coords + 3);
 }
 
 static void
@@ -1695,7 +1742,7 @@ cb_sheet_object_destroy (GtkObject *view, SheetObject *so)
 
 	if (scg) {
 		if (scg->current_object == so)
-			scg_mode_edit (scg);
+			scg_mode_edit ((SheetControl *) scg);
 		else
 			scg_object_stop_editing (scg, so);
 	}
@@ -1939,6 +1986,7 @@ int
 scg_colrow_distance_get (SheetControlGUI const *scg, gboolean is_cols,
 			 int from, int to)
 {
+	SheetControl *sc = (SheetControl *) scg;
 	ColRowCollection const *collection;
 	int default_size;
 	int i, pixels = 0;
@@ -1957,10 +2005,10 @@ scg_colrow_distance_get (SheetControlGUI const *scg, gboolean is_cols,
 
 	if (is_cols) {
 		g_return_val_if_fail (to <= SHEET_MAX_COLS, 1);
-		collection = &scg->sheet->cols;
+		collection = &sc->sheet->cols;
 	} else {
 		g_return_val_if_fail (to <= SHEET_MAX_ROWS, 1);
-		collection = &scg->sheet->rows;
+		collection = &sc->sheet->rows;
 	}
 
 	/* Do not use col_row_foreach, it ignores empties.
@@ -1992,38 +2040,44 @@ scg_colrow_distance_get (SheetControlGUI const *scg, gboolean is_cols,
 
 /*************************************************************************/
 
-void
-scg_cursor_bound (SheetControlGUI *scg, Range const *r)
+static void
+scg_cursor_bound (SheetControl *sc, Range const *r)
 {
-	gnumeric_sheet_cursor_bound (GNUMERIC_SHEET (scg->canvas), r);
+	SheetControlGUI *scg = (SheetControlGUI *) sc;
+
+	gnumeric_sheet_cursor_bound (GNUMERIC_SHEET (scg->gsheet), r);
 }
 
-void
-scg_compute_visible_region (SheetControlGUI *scg, gboolean full_recompute)
+static void
+scg_compute_visible_region (SheetControl *sc, gboolean full_recompute)
 {
-	gsheet_compute_visible_region (GNUMERIC_SHEET (scg->canvas),
+	SheetControlGUI *scg = (SheetControlGUI *) sc;
+
+	gsheet_compute_visible_region (GNUMERIC_SHEET (scg->gsheet),
 				       full_recompute);
 }
 
-void
-scg_make_cell_visible (SheetControlGUI  *scg, int col, int row,
+static void
+scg_make_cell_visible (SheetControl  *sc, int col, int row,
 		       gboolean force_scroll)
 {
-	gnumeric_sheet_make_cell_visible (GNUMERIC_SHEET (scg->canvas),
+	SheetControlGUI *scg = (SheetControlGUI *) sc;
+
+	gnumeric_sheet_make_cell_visible (GNUMERIC_SHEET (scg->gsheet),
 					  col, row, FALSE);
 }
 
 void
 scg_create_editor (SheetControlGUI *scg)
 {
-	gnumeric_sheet_create_editor (GNUMERIC_SHEET (scg->canvas));
+	gnumeric_sheet_create_editor (GNUMERIC_SHEET (scg->gsheet));
 }
 
 void
 scg_stop_editing (SheetControlGUI *scg)
 {
 	scg_rangesel_stop (scg, FALSE);
-	gnumeric_sheet_stop_editing (GNUMERIC_SHEET (scg->canvas));
+	gnumeric_sheet_stop_editing (GNUMERIC_SHEET (scg->gsheet));
 }
 
 /**
@@ -2040,6 +2094,7 @@ scg_rangesel_changed (SheetControlGUI *scg,
 	GnumericExprEntry *expr_entry;
 	gboolean ic_changed;
 	Range *r, last_r;
+	Sheet *sheet;
 
 	g_return_if_fail (IS_SHEET_CONTROL_GUI (scg));
 
@@ -2069,22 +2124,23 @@ scg_rangesel_changed (SheetControlGUI *scg,
 	 * We'll also need to name selections containing merged cells
 	 * properly.
 	 */
+	sheet = ((SheetControl *) scg)->sheet;
 	expr_entry = wbcg_get_entry_logical (scg->wbcg);
 	gnumeric_expr_entry_freeze (expr_entry);
 	ic_changed = gnumeric_expr_entry_set_rangesel_from_range (
-		expr_entry, r, scg->sheet, scg->rangesel.cursor_pos);
+		expr_entry, r, sheet, scg->rangesel.cursor_pos);
 	if (ic_changed)
 		gnumeric_expr_entry_get_rangesel (expr_entry, r, NULL);
 
 	last_r = *r;
-	sheet_merge_find_container (scg->sheet, r);
+	sheet_merge_find_container (sheet, r);
 	if (!range_equal (&last_r, r)) {
 		(void) gnumeric_expr_entry_set_rangesel_from_range (
-			expr_entry, r, scg->sheet, scg->rangesel.cursor_pos);
+			expr_entry, r, sheet, scg->rangesel.cursor_pos);
 		/* This can't grow the range further */
 	}
 	gnumeric_expr_entry_thaw (expr_entry);
-	gnumeric_sheet_rangesel_bound (GNUMERIC_SHEET (scg->canvas), r);
+	gnumeric_sheet_rangesel_bound (GNUMERIC_SHEET (scg->gsheet), r);
 }
 
 void
@@ -2100,9 +2156,10 @@ scg_rangesel_start (SheetControlGUI *scg, int col, int row)
 
 	scg->wbcg->rangesel = scg;
 	scg->rangesel.active = TRUE;
-	scg->rangesel.cursor_pos = GTK_EDITABLE (wbcg_get_entry_logical (scg->wbcg))->current_pos;
+	scg->rangesel.cursor_pos =
+		GTK_EDITABLE (wbcg_get_entry_logical (scg->wbcg))->current_pos;
 
-	gnumeric_sheet_rangesel_start (GNUMERIC_SHEET (scg->canvas), col, row);
+	gnumeric_sheet_rangesel_start (GNUMERIC_SHEET (scg->gsheet), col, row);
 	scg_rangesel_changed (scg, col, row, col, row);
 }
 
@@ -2118,7 +2175,7 @@ scg_rangesel_stop (SheetControlGUI *scg, gboolean clear_string)
 
 	scg->wbcg->rangesel = NULL;
 	scg->rangesel.active = FALSE;
-	gnumeric_sheet_rangesel_stop (GNUMERIC_SHEET (scg->canvas));
+	gnumeric_sheet_rangesel_stop (GNUMERIC_SHEET (scg->gsheet));
 	scg_stop_sliding (scg);
 	gnumeric_expr_entry_rangesel_stopped (
 		GNUMERIC_EXPR_ENTRY (wbcg_get_entry_logical (scg->wbcg)),
@@ -2140,7 +2197,7 @@ void
 scg_cursor_move_to (SheetControlGUI *scg, int col, int row,
 		    gboolean clear_selection)
 {
-	Sheet *sheet = scg->sheet;
+	Sheet *sheet = ((SheetControl *) scg)->sheet;
 
 	/*
 	 * Please note that the order here is important, as
@@ -2202,7 +2259,7 @@ void
 scg_rangesel_move (SheetControlGUI *scg, int n, gboolean jump_to_bound,
 		   gboolean horiz)
 {
-	Sheet const *sheet = scg->sheet;
+	Sheet *sheet = ((SheetControl *) scg)->sheet;
 	CellPos tmp;
 
 	if (!scg->rangesel.active)
@@ -2211,14 +2268,14 @@ scg_rangesel_move (SheetControlGUI *scg, int n, gboolean jump_to_bound,
 
 	tmp = scg->rangesel.base_corner;
 	if (horiz)
-		tmp.col = sheet_find_boundary_horizontal (scg->sheet,
-			tmp.col, tmp.row, tmp.row, n, jump_to_bound);
+		tmp.col = sheet_find_boundary_horizontal (
+			sheet, tmp.col, tmp.row, tmp.row, n, jump_to_bound);
 	else
-		tmp.row = sheet_find_boundary_vertical (scg->sheet,
-			tmp.col, tmp.row, tmp.col, n, jump_to_bound);
+		tmp.row = sheet_find_boundary_vertical (
+			sheet, tmp.col, tmp.row, tmp.col, n, jump_to_bound);
 
 	scg_rangesel_changed (scg, tmp.col, tmp.row, tmp.col, tmp.row);
-	gnumeric_sheet_make_cell_visible (GNUMERIC_SHEET (scg->canvas),
+	gnumeric_sheet_make_cell_visible (GNUMERIC_SHEET (scg->gsheet),
 					  tmp.col, tmp.row, FALSE);
 }
 
@@ -2226,22 +2283,24 @@ void
 scg_rangesel_extend (SheetControlGUI *scg, int n,
 		     gboolean jump_to_bound, gboolean horiz)
 {
+	Sheet *sheet = ((SheetControl *) scg)->sheet;
+	
 	if (scg->rangesel.active) {
 		CellPos tmp = scg->rangesel.move_corner;
 
 		if (horiz)
-			tmp.col = sheet_find_boundary_horizontal (scg->sheet,
+			tmp.col = sheet_find_boundary_horizontal (sheet,
 				tmp.col, tmp.row, scg->rangesel.base_corner.row,
 				n, jump_to_bound);
 		else
-			tmp.row = sheet_find_boundary_vertical (scg->sheet,
+			tmp.row = sheet_find_boundary_vertical (sheet,
 				tmp.col, tmp.row, scg->rangesel.base_corner.col,
 				n, jump_to_bound);
 
 		scg_rangesel_changed (scg,
 			scg->rangesel.base_corner.col,
 			scg->rangesel.base_corner.row, tmp.col, tmp.row);
-		gnumeric_sheet_make_cell_visible (GNUMERIC_SHEET (scg->canvas),
+		gnumeric_sheet_make_cell_visible (GNUMERIC_SHEET (scg->gsheet),
 			scg->rangesel.move_corner.col,
 			scg->rangesel.move_corner.row, FALSE);
 	} else
@@ -2263,7 +2322,7 @@ void
 scg_cursor_move (SheetControlGUI *scg, int n,
 		 gboolean jump_to_bound, gboolean horiz)
 {
-	Sheet *sheet = scg->sheet;
+	Sheet *sheet = ((SheetControl *) scg)->sheet;
 	CellPos tmp = sheet->edit_pos_real;
 
 	if (horiz)
@@ -2290,7 +2349,7 @@ void
 scg_cursor_extend (SheetControlGUI *scg, int n,
 		   gboolean jump_to_bound, gboolean horiz)
 {
-	Sheet *sheet = scg->sheet;
+	Sheet *sheet = ((SheetControl *) scg)->sheet;
 	CellPos tmp = sheet->cursor.move_corner;
 
 	if (horiz)
@@ -2312,5 +2371,33 @@ scg_take_focus (SheetControlGUI *scg)
 	g_return_if_fail (IS_SHEET_CONTROL_GUI (scg));
 
 	gtk_window_set_focus (wb_control_gui_toplevel (scg->wbcg),
-			      scg->canvas);
+			      scg->gsheet);
 }
+
+static void
+scg_class_init (GtkObjectClass *object_class)
+{
+	SheetControlClass *sc_class = SHEET_CONTROL_CLASS (object_class);
+
+	g_return_if_fail (sc_class != NULL);
+	
+	scg_parent_class = gtk_type_class (sheet_control_get_type ());
+	object_class->destroy = scg_destroy;
+
+	sc_class->resize                 = scg_resize;
+	sc_class->set_zoom_factor        = scg_set_zoom_factor;
+	sc_class->redraw_all             = scg_redraw_all;
+	sc_class->redraw_cell_region     = scg_redraw_cell_region;
+	sc_class->redraw_headers         = scg_redraw_headers;
+	sc_class->ant                    = scg_ant;
+	sc_class->unant                  = scg_unant;
+	sc_class->adjust_preferences     = scg_adjust_preferences;
+	sc_class->update_cursor_pos      = scg_update_cursor_pos;
+	sc_class->scrollbar_config       = scg_scrollbar_config;
+	sc_class->compute_visible_region = scg_compute_visible_region;
+	sc_class->make_cell_visible      = scg_make_cell_visible;
+	sc_class->cursor_bound           = scg_cursor_bound;
+}
+
+GNUMERIC_MAKE_TYPE (sheet_control_gui, "SheetControlGUI", SheetControlGUI,
+		    scg_class_init, scg_init, sheet_control_get_type ())
